@@ -1,16 +1,16 @@
-/* $Id: astarnode.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: astarnode.cpp 32765 2009-02-11 23:04:59Z shadowmaster $ */
 /*
-Copyright (C) 2003 by David White <dave@whitevine.net>
-Copyright (C) 2005 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-Part of the Battle for Wesnoth Project http://www.wesnoth.org/
+   Copyright (C) 2003 by David White <dave@whitevine.net>
+                 2005 - 2009 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License version 2
-or at your option any later version.
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
 
-See the COPYING file for more details.
+   See the COPYING file for more details.
 */
 
 #include "global.hpp"
@@ -18,8 +18,8 @@ See the COPYING file for more details.
 
 #include <cassert>
 
-void a_star_node::initNode(gamemap::location const &pos, gamemap::location const &dst,
-                           double cost, a_star_node *parent, std::set<gamemap::location> const *teleports)
+void a_star_node::initNode(map_location const &pos, map_location const &dst,
+                           double cost, a_star_node *parent, std::set<map_location> const *teleports)
 {
 	isInCloseList = false;
 	loc = pos;
@@ -29,7 +29,7 @@ void a_star_node::initNode(gamemap::location const &pos, gamemap::location const
 	//if there are teleport locations, correct the heuristic to take them into account
 	if (teleports != NULL) {
 		double srch = h, dsth = h;
-		std::set<gamemap::location>::const_iterator i;
+		std::set<map_location>::const_iterator i;
 		for(i = teleports->begin(); i != teleports->end(); ++i) {
 			const double new_srch = heuristic(pos, *i);
 			const double new_dsth = heuristic(*i, dst);
@@ -69,10 +69,12 @@ void a_star_world::poss_a_star_node::addPage()
 	capacity_ += nbElemByPage_;
 }
 
-a_star_world::poss_a_star_node::poss_a_star_node()
-	: capacity_(0), curIndex_(0)
+a_star_world::poss_a_star_node::poss_a_star_node() :
+		vectPageAStarNode_(),
+		nbElemByPage_(size_t((4096 - 24) / sizeof(a_star_node))),
+		capacity_(0),
+		curIndex_(0)
 {
-	nbElemByPage_ = size_t((4096 - 24) / sizeof(a_star_node));
 	assert(nbElemByPage_ > 0);
 	addPage();
 }
@@ -114,8 +116,11 @@ void a_star_world::poss_a_star_node::clear()
 	//---------------------------------------------------
 }
 
-a_star_world::a_star_world()
-	: pool_(new poss_a_star_node), width_(0), nbNode_(0)
+a_star_world::a_star_world() :
+		pool_(new poss_a_star_node),
+		vectAStarNode_(),
+		width_(0),
+		nbNode_(0)
 {
 }
 
@@ -137,7 +142,7 @@ void a_star_world::resize_IFN(size_t parWidth, size_t parHeight)
 	vectAStarNode_.resize(sz);
 }
 
-void a_star_world::clear(void)
+void a_star_world::clear()
 {
 	a_star_node *locNode = NULL;
 	std::fill(vectAStarNode_.begin(), vectAStarNode_.end(), locNode);
@@ -145,7 +150,7 @@ void a_star_world::clear(void)
 	pool_->clear();
 }
 
-a_star_node *a_star_world::getNodeFromLocation(gamemap::location const &loc, bool &isCreated)
+a_star_node *a_star_world::getNodeFromLocation(map_location const &loc, bool &isCreated)
 {
 	//----------------- PRE_CONDITIONS ------------------
 	assert(loc.valid());
