@@ -1,4 +1,4 @@
-/* $Id: attack_prediction.hpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: attack_prediction.hpp 34271 2009-03-29 23:01:10Z dfranke $ */
 /*
    Copyright (C) 2007 - 2009
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -33,7 +33,7 @@ struct combatant
 	combatant(const combatant &that, const battle_context::unit_stats &u);
 
 	/** Simulate a fight!  Can be called multiple times for cumulative calculations. */
-	void fight(combatant &opponent);
+	void fight(combatant &opponent,bool levelup_considered=true);
 
 	/** takes into account level up when calculating resulting HP after a fight */
 	void consider_levelup(combatant &opponent);
