@@ -1,6 +1,6 @@
-/* $Id: color_range.cpp 26448 2008-05-07 17:35:33Z mordante $ */
+/* $Id: color_range.cpp 31858 2009-01-01 10:27:41Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,21 +12,17 @@
    See the COPYING file for more details.
 */
 
-//! @file color_range.cpp
-//! Generate ranges of colors, and color palettes.
-//! Used e.g. to color HP, XP.
+/**
+ * @file color_range.cpp
+ * Generate ranges of colors, and color palettes.
+ * Used e.g. to color HP, XP.
+ */
 
 #include "game_config.hpp"
 #include "global.hpp"
-#include "color_range.hpp"
 #include "map.hpp"
-#include "serialization/string_utils.hpp"
 
 #include <cassert>
-#include <set>
-#include <string>
-#include <cstring>
-#include <vector>
 
 std::map<Uint32, Uint32> recolor_range(const color_range& new_range, const std::vector<Uint32>& old_rgb){
 	std::map<Uint32, Uint32> map_rgb;
@@ -65,8 +61,8 @@ std::map<Uint32, Uint32> recolor_range(const color_range& new_range, const std::
 			new_g=Uint32( old_rat * new_green + (1 - old_rat) * min_green);
 			new_b=Uint32( old_rat * new_blue  + (1 - old_rat) * min_blue);
 		}else if(255 - reference_avg){
-			float old_rat = (255.0 - static_cast<float>(old_avg)) /
-				(255.0 - reference_avg);
+			float old_rat = (255.0f - static_cast<float>(old_avg)) /
+				(255.0f - reference_avg);
 
 			new_r=static_cast<Uint32>( old_rat * new_red   + (1 - old_rat) * max_red);
 			new_g=static_cast<Uint32>( old_rat * new_green + (1 - old_rat) * max_green);
@@ -89,7 +85,6 @@ std::map<Uint32, Uint32> recolor_range(const color_range& new_range, const std::
 	return map_rgb;
 }
 
-//! Convert comma separated string into rgb values.
 std::vector<Uint32> string2rgb(std::string s){
 	std::vector<Uint32> out;
 	std::vector<std::string> rgb_vec = utils::split(s);
