@@ -1,6 +1,6 @@
-/* $Id: config.hpp 24568 2008-03-12 15:14:54Z soliton $ */
+/* $Id: config.hpp 31858 2009-01-01 10:27:41Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,34 +12,44 @@
    See the COPYING file for more details.
 */
 
-//! @file config.hpp
-//! Definitions for the interface to Wesnoth Markup Language (WML).
+/**
+ * @file config.hpp
+ * Definitions for the interface to Wesnoth Markup Language (WML).
+ *
+ * This module defines the interface to Wesnoth Markup Language (WML).  WML is
+ * a simple hierarchical text-based file format.  The format is defined in
+ * Wiki, under BuildingScenariosWML
+ *
+ * All configuration files are stored in this format, and data is sent across
+ * the network in this format.  It is thus used extensively throughout the
+ * game.
+ */
 
 #ifndef CONFIG_HPP_INCLUDED
 #define CONFIG_HPP_INCLUDED
 
 #include "global.hpp"
 
+#include <boost/shared_ptr.hpp>
+
 #include <map>
-#include <memory> 
+#include <memory>
 #include <ostream>
 #include <string>
 #include <vector>
+
+#include "game_errors.hpp"
 #include "tstring.hpp"
 #include "serialization/string_utils.hpp"
 
 
-// This module defines the interface to Wesnoth Markup Language (WML).
-// WML is a simple hierarchical text-based file format.
-// The format is defined in Wiki, under BuildingScenariosWML
-//
-// All configuration files are stored in this format,
-// and data is sent across the network in this format.
-// It is thus used extensively throughout the game.
-
 typedef std::map<std::string,t_string> string_map;
 
-//! A config object defines a single node in a WML file, with access to child nodes.
+class config;
+
+typedef boost::shared_ptr<config> config_ptr;
+
+/** A config object defines a single node in a WML file, with access to child nodes. */
 class config
 {
 public:
@@ -47,7 +57,7 @@ public:
 	config();
 
 	config(const config& cfg);
-	
+
 	// Create a config with an empty child of name 'child'.
 	config(const std::string& child);
 	~config();
@@ -65,6 +75,7 @@ public:
 
 	child_itors child_range(const std::string& key);
 	const_child_itors child_range(const std::string& key) const;
+	size_t child_count(const std::string& key) const;
 
 	const child_list& get_children(const std::string& key) const;
 	const child_map& all_children() const;
@@ -79,6 +90,17 @@ public:
 
 	const t_string& get_attribute(const std::string& key) const;
 	bool has_attribute(const std::string& key) const {return values.find(key) != values.end();}
+	void remove_attribute(const std::string& key) {values.erase(key);}
+
+	/**
+	 * This should only be used if there is no mapping of the key already,
+	 * e.g. when creating a new config object. It does not replace an existing value.
+	 * @returns true when it added the key-value pair, false if it already existed
+	 *          (and you probably wanted to use config[key] = value)
+	 */
+	bool init_attribute(const std::string& key, const t_string& value) {
+		return values.insert(string_map::value_type(key, value)).second;
+	}
 
 	config* find_child(const std::string& key, const std::string& name,
 	                   const t_string& value);
@@ -95,9 +117,8 @@ public:
 	std::string debug() const;
 	std::string hash() const;
 
-	struct error {
-		error(const std::string& msg) : message(msg) {}
-		std::string message;
+	struct error : public game::error {
+		error(const std::string& message) : game::error(message) {}
 	};
 
 	struct child_pos {
@@ -135,48 +156,64 @@ public:
 		Itor i_;
 	};
 
-	//! In-order iteration over all children.
+	/** In-order iteration over all children. */
 	all_children_iterator ordered_begin() const;
 	all_children_iterator ordered_end() const;
 	all_children_iterator erase(const all_children_iterator& i);
 
-	//! A function to get the differences between this object,
-	//! and 'c', as another config object.
-	//! I.e. calling cfg2.apply_diff(cfg1.get_diff(cfg2))
-	//! will make cfg1 identical to cfg2.
+	/**
+	 * A function to get the differences between this object,
+	 * and 'c', as another config object.
+	 * I.e. calling cfg2.apply_diff(cfg1.get_diff(cfg2))
+	 * will make cfg1 identical to cfg2.
+	 */
 	config get_diff(const config& c) const;
 	void get_diff(const config& c, config& res) const;
 
 	void apply_diff(const config& diff); //throw error
 
-	//! Merge config 'c' into this config.
-	//! Overwrites this config's values.
+	/**
+	 * Merge config 'c' into this config.
+	 * Overwrites this config's values.
+	 */
 	void merge_with(const config& c);
 
+	/**
+	 * Merge config 'c' into this config.
+	 * Keeps this config's values and does not add existing elements.
+	 * NOTICE: other nonstandard behavior includes no merge recursion into child
+	 * and has limited merging for child lists of differing lengths
+	 */
+    void merge_and_keep(const config& c);
+
 	bool matches(const config &filter) const;
 
-	//! Removes keys with empty values.
+	/** Removes keys with empty values. */
 	void prune();
 
-	//! Append data from another config object to this one.
-	//! Attributes in the latter config object will clobber attributes in this one.
+	/**
+	 * Append data from another config object to this one.
+	 * Attributes in the latter config object will clobber attributes in this one.
+	 */
 	void append(const config& cfg);
 
-	//! All children with the given key will be merged
-	//! into the first element with that key.
+	/**
+	 * All children with the given key will be merged
+	 * into the first element with that key.
+	 */
 	void merge_children(const std::string& key);
 
-	//! Resets the translated values of all strings contained in this object
+	/** Resets the translated values of all strings contained in this object */
 	void reset_translation() const;
 
 	//this is a cheap O(1) operation
 	void swap(config& cfg);
 
-	//! All the attributes of this node.
+	/** All the attributes of this node. */
 	string_map values;
 
 private:
-	//! A list of all children of this node.
+	/** A list of all children of this node. */
 	child_map children;
 
 	std::vector<child_pos> ordered_children;
