@@ -1,4 +1,4 @@
-/* $Id: display.hpp 33767 2009-03-17 19:19:34Z alink $ */
+/* $Id: display.hpp 35509 2009-05-09 16:25:14Z silene $ */
 /*
    Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -276,6 +276,9 @@ public:
 	/** check if an hexes under the rectangle is invalidated */
 	bool rectangle_need_update(const SDL_Rect& rect) const;
 
+	/** check if an hex is invalidated */
+	bool hex_need_update(const map_location& loc) const;
+
 	/**
 	 * Function to invalidate animated terrains which may have changed.
 	 */
@@ -431,6 +434,9 @@ public:
 	/** Check if a tile is fully on screen. */
 	bool tile_on_screen(const map_location& loc);
 
+	/** Checks if location @a loc or one of the adjacent tiles is visible on screen. */
+	bool tile_nearly_on_screen(const map_location &loc);
+
 	/**
 	 * Draws invalidated items.
 	 * If update is true, will also copy the display to the frame buffer.
