@@ -1,6 +1,6 @@
-/* $Id: events.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: events.hpp 31858 2009-01-01 10:27:41Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,9 @@
 
 //our user-defined double-click event type
 #define DOUBLE_CLICK_EVENT SDL_USEREVENT
+#define HOVER_EVENT (SDL_USEREVENT + 1)
+#define HOVER_REMOVE_POPUP_EVENT (SDL_USEREVENT + 2)
+#define DRAW_EVENT (SDL_USEREVENT + 3)
 
 namespace events
 {
