@@ -1,4 +1,4 @@
-/* $Id: filesystem.cpp 33585 2009-03-13 00:28:36Z soliton $ */
+/* $Id: filesystem.cpp 34025 2009-03-22 12:39:12Z mordante $ */
 /*
    Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -680,7 +680,11 @@ void write_file(const std::string& fname, const std::string& data)
 
 std::string read_map(const std::string& name)
 {
-	std::string res = read_file(get_wml_location("maps/" + name));
+	std::string res;
+	std::string map_location = get_wml_location("maps/" + name);
+	if(!map_location.empty()) {
+		res = read_file(map_location);
+	}
 
 	if (res.empty()) {
 		res = read_file(get_user_data_dir() + "/editor/maps/" + name);
