@@ -1,6 +1,6 @@
-/* $Id: floating_textbox.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: floating_textbox.cpp 31859 2009-01-01 10:28:26Z mordante $ */
 /*
-   Copyright (C) 2006 - 2008 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playturn Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -16,19 +16,17 @@
 #include "global.hpp"
 
 #include "floating_textbox.hpp"
-#include "font.hpp"
 #include "game_display.hpp"
 #include "game_preferences.hpp"
 #include "log.hpp"
-#include "team.hpp"
 
 #include <ctime>
 
 namespace gui{
-	floating_textbox::floating_textbox() : 
-		box_(NULL), 
-		check_(NULL), 
-		mode_(TEXTBOX_NONE), 
+	floating_textbox::floating_textbox() :
+		box_(NULL),
+		check_(NULL),
+		mode_(TEXTBOX_NONE),
 		label_string_(),
 		label_(0)
 	{}
