@@ -1,4 +1,4 @@
-/* $Id: game.cpp 33699 2009-03-15 19:08:36Z shadowmaster $ */
+/* $Id: game.cpp 37235 2009-07-26 11:02:25Z ilor $ */
 /*
    Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -61,6 +61,7 @@
 #include "thread.hpp"
 #include "titlescreen.hpp"
 #include "upload_log.hpp"
+#include "version.hpp"
 #include "wml_exception.hpp"
 #include "wml_separators.hpp"
 #include "serialization/binary_or_text.hpp"
@@ -394,12 +395,23 @@ game_controller::game_controller(int argc, char** argv) :
 #ifndef DISABLE_EDITOR2
 		} else if(val == "-e" || val == "--editor") {
 			jump_to_editor_ = true;
+			if(arg_+1 != argc_) {
+				if (argv_[arg_ + 1][0] != '-') {
+					++arg_;
+					loaded_game_ = argv_[arg_];
+				}
+			}
 #endif
 		} else if(val == "--dummylocales") {
-			std::cerr << "--dummylocales is deprecated use --dummy-locales instead.\n";
-			game_config::use_dummylocales = true;
+			std::cerr << "Option --dummylocales is deprecated; use --dummy-locales instead.\n";
+			goto enable_dummy_locales;
 		} else if(val == "--dummy-locales") {
+			enable_dummy_locales:
+#ifdef USE_DUMMYLOCALES
 			game_config::use_dummylocales = true;
+#else
+			std::cerr << "Option --dummy-locales ignored: support was not compiled in.\n";
+#endif
 		} else if(val[0] == '-') {
 			std::cerr << "unknown option: " << val << std::endl;
 			throw config::error("unknown option");
@@ -632,14 +644,13 @@ bool game_controller::play_test()
 
 	state_.campaign_type = "test";
 	state_.scenario = test_scenario_;
+	state_.campaign_define = "TEST";
+	cache_.clear_defines();
+	cache_.add_define("TEST");
 
-	try {
-		load_game_cfg();
-	} catch(config::error&) {
-		cache_.clear_defines();
-		load_game_cfg();
-		return false;
-	}
+	load_game_cfg();
+
+	paths_manager_.set_paths(game_config_);
 
 	try {
 		upload_log nolog(false);
@@ -837,6 +848,7 @@ bool game_controller::play_multiplayer_mode()
 			ai_params[j->first] = j->second;
 		}
 	}
+	level.add_child("era", *era_cfg);
 
 	try {
 		upload_log nolog(false);
@@ -929,23 +941,29 @@ bool game_controller::load_game()
 
 		const std::string version = cfg["version"];
 		if(version != game_config::version) {
-			// do not load if too old, if either the savegame or the current game
-			// has the version 'test' allow loading
-			if(!game_config::is_compatible_savegame_version(version)) {
-				/* GCC-3.3 needs a temp var otherwise compilation fails */
-				gui::message_dialog dlg(disp(), "", _("This save is from a version too old to be loaded."));
-				dlg.show();
-				return false;
-			}
+			const version_info parsed_savegame_version(version);
+			if(game_config::wesnoth_version.minor_version() % 2 != 0 ||
+			   game_config::wesnoth_version.major_version() != parsed_savegame_version.major_version() ||
+			   game_config::wesnoth_version.minor_version() != parsed_savegame_version.minor_version()) {
+				// do not load if too old, if either the savegame or the current game
+				// has the version 'test' allow loading
+				if(!game_config::is_compatible_savegame_version(version)) {
+					/* GCC-3.3 needs a temp var otherwise compilation fails */
+					gui::message_dialog dlg(disp(), "", _("This save is from a version too old to be loaded."));
+					dlg.show();
+					return false;
+				}
 
-			const int res = gui::dialog(disp(),"",
-			                      _("This save is from a different version of the game. Do you want to try to load it?"),
-			                      gui::YES_NO).show();
-			if(res == 1) {
-				return false;
+				const int res = gui::dialog(disp(),"",
+									_("This save is from a different version of the game. Do you want to try to load it?"),
+									gui::YES_NO).show();
+				if(res == 1) {
+					return false;
+				}
 			}
 		}
 
+		paths_manager_.set_paths(game_config_);
 		state_ = game_state(cfg, show_replay);
 
 		// Get the status of the random in the snapshot.
@@ -1265,10 +1283,10 @@ void game_controller::start_wesnothd()
 			iname != paths_to_try.end(); ++iname)
 	{
 #ifndef _WIN32
-		std::string command = "\"" + *iname +"\" -c " + config + " -d -t 2 -T 5 ";
+		std::string command = "\"" + *iname +"\" -c \"" + config + "\" -d -t 2 -T 5";
 #else
 		// start wesnoth as background job
-		std::string command = "cmd /C start \"wesnoth server\" /B \"" + *iname + "\" -c " + config + " -t 2 -T 5 ";
+		std::string command = "cmd /C start \"wesnoth server\" /B \"" + *iname + "\" -c \"" + config + "\" -t 2 -T 5";
 #endif
 		LOG_GENERAL << "Starting wesnothd: "<< command << "\n";
 		if (std::system(command.c_str()) == 0)
@@ -1476,6 +1494,7 @@ void game_controller::reset_translations()
 		// we may have translatable strings in [game_config]
 		// e.g. team color names are defined there
 		game_config::load_config(game_config_.child("game_config"));
+		set_unit_data();
 	} catch(game::error& e) {
 		ERR_CONFIG << "Error loading game configuration files\n";
 		gui::show_error_message(disp(), _("Error loading game configuration files: '") +
@@ -1790,7 +1809,8 @@ static int process_command_args(int argc, char** argv) {
 			<< "  --dummy-locales              enables dummy locales for switching to non-system\n"
 			<< "                               locales.\n"
 #ifndef DISABLE_EDITOR2
-			<< "  -e, --editor                 starts the in-game map editor directly.\n"
+			<< "  -e, --editor [<file>]        starts the in-game map editor directly. If <file>\n"
+			<< "                               is specified, equivalent to -e --load <file>.\n"
 #endif
 			<< "  --fps                        displays the number of frames per second the\n"
 			<< "                               game is currently running at, in a corner of\n"
@@ -1807,7 +1827,8 @@ static int process_command_args(int argc, char** argv) {
 #ifndef DISABLE_EDITOR2
 			<< "                               When launching the map editor via -e, the map\n"
 			<< "                               <file> is loaded, relative to the current\n"
-			<< "                               directory.\n"
+			<< "                               directory. If it is a directory, the editor\n"
+			<< "                               will start with a load map dialog opened there.\n"
 #endif
 			<< "  --log-<level>=<domain1>,<domain2>,...\n"
 			<< "                               sets the severity level of the log domains.\n"
@@ -2143,6 +2164,7 @@ static int do_gameloop(int argc, char** argv)
 		} else if(res == gui::RELOAD_GAME_DATA) {
 			loadscreen::global_loadscreen_manager loadscreen(game.disp().video());
 			game.reload_changed_game_config();
+			image::flush_cache();
 			continue;
 #ifndef DISABLE_EDITOR2
 		} else if(res == gui::START_MAP_EDITOR) {
