@@ -1,4 +1,4 @@
-/* $Id: game_display.cpp 33504 2009-03-09 23:19:15Z alink $ */
+/* $Id: game_display.cpp 35509 2009-05-09 16:25:14Z silene $ */
 /*
    Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -885,18 +885,24 @@ void game_display::invalidate_animations()
 {
 	new_animation_frame();
 	display::invalidate_animations();
-	unit_map::iterator unit;
-	for(unit=units_.begin() ; unit != units_.end() ; unit++)
-		unit->second.refresh(*this, unit->first);
-	if (temp_unit_ )
+	for (unit_map::iterator u = units_.begin(),
+	     u_end = units_.end(); u != u_end; ++u)
+	{
+		if (!tile_nearly_on_screen(u->first)) continue;
+		u->second.refresh(*this, u->first);
+	}
+	if (temp_unit_ && tile_nearly_on_screen(temp_unit_loc_))
 		temp_unit_->refresh(*this, temp_unit_loc_);
 	bool new_inval = true;
 	while(new_inval) {
 		new_inval = false;
-		for(unit=units_.begin() ; unit != units_.end() ; unit++) {
-			new_inval |= unit->second.invalidate(unit->first);
+		for (unit_map::iterator u = units_.begin(),
+		     u_end = units_.end(); u != u_end; ++u)
+		{
+			if (!tile_nearly_on_screen(u->first)) continue;
+			new_inval |= u->second.invalidate(u->first);
 		}
-		if (temp_unit_ ) {
+		if (temp_unit_ && tile_nearly_on_screen(temp_unit_loc_)) {
 			//new_inval |=invalidate(temp_unit_loc_);
 			new_inval |=temp_unit_->invalidate(temp_unit_loc_);
 		}
