@@ -1,4 +1,4 @@
-/* $Id: game_preferences_display.cpp 33157 2009-02-28 16:12:49Z ivanovic $ */
+/* $Id: game_preferences_display.cpp 36370 2009-06-23 19:40:55Z mordante $ */
 /*
    Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -1204,7 +1204,9 @@ std::string show_wesnothd_server_search(display& disp)
 
 	symbols["filename"] = filename;
 
-	const std::string title =  vgettext("Find $filename server binary to host networked games", symbols);
+	const std::string title = utils::interpolate_variables_into_string(
+			  _("Find $filename server binary to host networked games")
+			, &symbols);
 
 	int res = dialogs::show_file_chooser_dialog(disp, path, title, false, filename);
 	if (res == 0)
