@@ -1,4 +1,4 @@
-/* $Id: scrollbar_container.cpp 33777 2009-03-17 20:41:59Z mordante $ */
+/* $Id: scrollbar_container.cpp 37939 2009-08-18 19:47:08Z mordante $ */
 /*
    copyright (C) 2008 - 2009 by mark de wever <koraq@xs4all.nl>
    part of the battle for wesnoth project http://www.wesnoth.org/
@@ -922,5 +922,11 @@ void tscrollbar_container::scrollbar_moved()
 	set_scrollbar_button_status();
 }
 
+const std::string& tscrollbar_container::get_control_type() const
+{
+	static const std::string type = "scrollbar_container";
+	return type;
+}
+
 } // namespace gui2
 
