@@ -1,4 +1,4 @@
-/* $Id: spacer.cpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: spacer.cpp 37939 2009-08-18 19:47:08Z mordante $ */
 /*
    copyright (C) 2008 - 2009 by mark de wever <koraq@xs4all.nl>
    part of the battle for wesnoth project http://www.wesnoth.org/
@@ -14,10 +14,16 @@
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
+#include "gui/widgets/spacer.hpp"
+
 namespace gui2 {
 
+const std::string& tspacer::get_control_type() const
+{
+	static const std::string type = "spacer";
+	return type;
+}
 
 } // namespace gui2
 
 
-
