@@ -1,10 +1,10 @@
-/* $Id: leader_list.cpp 24046 2008-02-25 17:29:06Z noyga $ */
+/* $Id: leader_list.cpp 33579 2009-03-12 22:04:22Z shadowmaster $ */
 /*
-   Copyright (C) 2007 - 2008
+   Copyright (C) 2007 - 2009
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License
+   it under the terms of the GNU General Public License 2
    or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
@@ -12,29 +12,29 @@
    See the COPYING file for more details.
 */
 
-//! @file leader_list.cpp 
-//! Manage the selection of a leader, and select his/her gender.
+/**
+ * @file leader_list.cpp
+ * Manage the selection of a leader, and select his/her gender.
+ */
 
 #include "global.hpp"
 
 #include "gettext.hpp"
 #include "leader_list.hpp"
 #include "wml_separators.hpp"
-#include "serialization/string_utils.hpp"
 #include "widgets/combo.hpp"
-#include "widgets/menu.hpp"
 
-const std::string leader_list_manager::random_enemy_picture("random-dice.png~RC(magenta>1)");
+const std::string leader_list_manager::random_enemy_picture("units/random-dice.png");
 
 leader_list_manager::leader_list_manager(const config::child_list& side_list,
-		const game_data* data, gui::combo* leader_combo , gui::combo* gender_combo):
+		gui::combo* leader_combo , gui::combo* gender_combo):
 	leaders_(),
 	genders_(),
 	gender_ids_(),
-	side_list_(side_list), 
-	data_(data),
-	leader_combo_(leader_combo), 
-	gender_combo_(gender_combo)
+	side_list_(side_list),
+	leader_combo_(leader_combo),
+	gender_combo_(gender_combo),
+	colour_(0)
 {
 }
 
@@ -69,7 +69,7 @@ void leader_list_manager::update_leader_list(int side_index)
 
 	leaders_.clear();
 
-	if(side["random_faction"] == "yes") {
+	if(utils::string_bool(side["random_faction"])) {
 		if(leader_combo_ != NULL) {
 			std::vector<std::string> dummy;
 			dummy.push_back("-");
@@ -126,11 +126,11 @@ void leader_list_manager::update_gender_list(const std::string& leader)
 		return;
 	}
 
-	const game_data::unit_type_map& utypes = data_->unit_types;
-	if (utypes.find(leader) != utypes.end()) {
+	unit_type_data::unit_type_map_wrapper& utypes = unit_type_data::types();
+	if (utypes.unit_type_exists(leader)) {
 		const unit_type* ut;
 		const unit_type* utg;
-		ut = &(utypes.find(leader)->second);
+		ut = &(utypes.find_unit_type(leader)->second);
 		const std::vector<unit_race::GENDER> genders = ut->genders();
 		if ( (genders.size() < 2) && (gender_combo_ != NULL) ) {
 			gender_combo_->enable(false);
@@ -143,29 +143,21 @@ void leader_list_manager::update_gender_list(const std::string& leader)
 			utg = &(ut->get_gender_unit_type(*i));
 
 			// Make the internationalized titles for each gender, along with the WML ids
-			if (*i == unit_race::FEMALE)
-			{
+			if (*i == unit_race::FEMALE) {
 				gender_ids_.push_back("female");
-#ifdef LOW_MEM
-				genders_.push_back(IMAGE_PREFIX + utg->image() + COLUMN_SEPARATOR + _("Female ♀"));
-#else
-				genders_.push_back(IMAGE_PREFIX + utg->image() + std::string("~RC(" + utg->flag_rgb() + ">1)") + COLUMN_SEPARATOR + _("Female ♀"));
-#endif
-			}
-			else
-			{
+				genders_.push_back(IMAGE_PREFIX + utg->image() + get_RC_suffix(utg->flag_rgb()) +
+						COLUMN_SEPARATOR + _("Female ♀"));
+			} else {
 				gender_ids_.push_back("male");
-#ifdef LOW_MEM
-				genders_.push_back(IMAGE_PREFIX + utg->image() + COLUMN_SEPARATOR + _("Male ♂"));
-#else
-				genders_.push_back(IMAGE_PREFIX + utg->image() + std::string("~RC(" + utg->flag_rgb() + ">1)") + COLUMN_SEPARATOR + _("Male ♂"));
-#endif
+				genders_.push_back(IMAGE_PREFIX + utg->image() + get_RC_suffix(utg->flag_rgb()) +
+						COLUMN_SEPARATOR + _("Male ♂"));
 			}
 		}
 		if (gender_combo_ != NULL) {
 			gender_combo_->set_items(genders_);
+			assert(genders_.size()!=0);
 			gender_index %= genders_.size();
-			gender_combo_->set_selected(gender_index);	
+			gender_combo_->set_selected(gender_index);
 		}
 	} else {
 		gender_ids_.push_back("random");
@@ -173,7 +165,7 @@ void leader_list_manager::update_gender_list(const std::string& leader)
 		if (gender_combo_ != NULL) {
 			gender_combo_->enable(false);
 			gender_combo_->set_items(genders_);
-			gender_combo_->set_selected(0);	
+			gender_combo_->set_selected(0);
 		}
 	}
 }
@@ -183,27 +175,23 @@ void leader_list_manager::populate_leader_combo(int selected_index) {
 	std::vector<std::string> leader_strings;
 	for(itor = leaders_.begin(); itor != leaders_.end(); ++itor) {
 
-		const game_data::unit_type_map& utypes = data_->unit_types;
+		unit_type_data::unit_type_map_wrapper& utypes = unit_type_data::types();
 
-		//const std::string name = data_->unit_types->find(*itor).language_name();
-		if (utypes.find(*itor) != utypes.end()) {
+		//const std::string name = data_->unit_types->find(*itor).type_name();
+		if (utypes.unit_type_exists(*itor)) {
 			const unit_type* ut;
-			ut = &(utypes.find(*itor)->second);
+			ut = &(utypes.find_unit_type(*itor)->second);
 			if (gender_combo_ != NULL && !genders_.empty() && size_t(gender_combo_->selected()) < genders_.size()) {
 				if (gender_ids_[gender_combo_->selected()] == "male"){
-					ut = &(utypes.find(*itor)->second.get_gender_unit_type(unit_race::MALE));
+					ut = &(utypes.find_unit_type(*itor)->second.get_gender_unit_type(unit_race::MALE));
 				} else if (gender_ids_[gender_combo_->selected()] == "female") {
-					ut = &(utypes.find(*itor)->second.get_gender_unit_type(unit_race::FEMALE));
+					ut = &(utypes.find_unit_type(*itor)->second.get_gender_unit_type(unit_race::FEMALE));
 				}
 			}
-			const std::string name =  ut->language_name();
+			const std::string name =  ut->type_name();
 			const std::string image = ut->image();
+			leader_strings.push_back(IMAGE_PREFIX + image + get_RC_suffix(ut->flag_rgb()) + COLUMN_SEPARATOR + name);
 
-#ifdef LOW_MEM
-			leader_strings.push_back(IMAGE_PREFIX + image + COLUMN_SEPARATOR + name);
-#else
-			leader_strings.push_back(IMAGE_PREFIX + image + std::string("~RC(" + ut->flag_rgb() + ">1)") + COLUMN_SEPARATOR + name);
-#endif
 		} else {
 			if(*itor == "random") {
 				leader_strings.push_back(IMAGE_PREFIX + random_enemy_picture + COLUMN_SEPARATOR + _("Random"));
@@ -271,3 +259,11 @@ std::string leader_list_manager::get_gender() const
 		return "null";
 	return gender_ids_[gender_combo_->selected()];
 }
+
+std::string leader_list_manager::get_RC_suffix(const std::string& unit_colour) const {
+#ifdef LOW_MEM
+	return "";
+#else
+ 	return "~RC("+unit_colour+">"+lexical_cast<std::string>(colour_+1) +")";
+#endif
+}
