@@ -1,4 +1,4 @@
-/* $Id: map.cpp 32980 2009-02-21 20:48:26Z ilor $ */
+/* $Id: map.cpp 36510 2009-07-02 10:16:13Z soliton $ */
 /*
    Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -212,9 +212,11 @@ void gamemap::read(const std::string& data)
 		// the first valid position is 1,
 		// so the offset 0 in the array is never used.
 		if(itor->first < 1 || itor->first >= MAX_PLAYERS+1) {
-			ERR_CF << "Starting position " << itor->first << " out of range\n";
-			throw incorrect_map_format_exception("Illegal starting position found"
-				" in map. The scenario cannot be loaded.");
+			std::stringstream ss;
+			ss << "Starting position " << itor->first << " out of range\n";
+			ERR_CF << ss.str();
+			ss << "The map cannot be loaded.";
+			throw incorrect_map_format_exception(ss.str().c_str());
 		}
 
 		// Add to the starting position array
@@ -233,9 +235,12 @@ void gamemap::read(const std::string& data)
 			// Is the terrain valid?
 			if(tcodeToTerrain_.count(tiles_[x][y]) == 0) {
 				if(!try_merge_terrains(tiles_[x][y])) {
-					ERR_CF << "Illegal character in map: (" << t_translation::write_terrain_code(tiles_[x][y])
+					std::stringstream ss;
+					ss << "Illegal character in map: (" << t_translation::write_terrain_code(tiles_[x][y])
 						   << ") '" << tiles_[x][y] << "'\n";
-					throw incorrect_map_format_exception("Illegal character found in map. The scenario cannot be loaded.");
+					ERR_CF << ss.str();
+					ss << "The map cannot be loaded.";
+					throw incorrect_map_format_exception(ss.str().c_str());
 				}
 			}
 
