@@ -1,6 +1,6 @@
-/* $Id: map_create.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: map_create.cpp 31858 2009-01-01 10:27:41Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,17 +13,13 @@
 */
 
 #include "global.hpp"
+#include "map_create.hpp"
 
 #include "cavegen.hpp"
 #include "log.hpp"
-#include "map_create.hpp"
 #include "mapgen_dialog.hpp"
 #include "scoped_resource.hpp"
-#include "serialization/string_utils.hpp"
 
-#include <ostream>
-#include <string>
-#include <vector>
 
 #define ERR_CF LOG_STREAM(err, config)
 
