@@ -1,6 +1,6 @@
-/* $Id: map_label.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: map_label.cpp 32329 2009-01-27 16:50:09Z zookeeper $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -15,12 +15,9 @@
 #include "global.hpp"
 
 #include "display.hpp"
-#include "font.hpp"
-#include "language.hpp"
 #include "map_label.hpp"
+#include "formula_string_utils.hpp"
 
-#include <cassert>
-#include <vector>
 
 namespace {
 	const size_t max_label_size = 32;
@@ -30,28 +27,28 @@ namespace {
 //or the tile below is obscured. This is because in the case where the tile
 //itself is visible, but the tile below is obscured, the bottom half of the
 //tile will still be shrouded, and the label being drawn looks weird
-static bool is_shrouded(const display& disp, const gamemap::location& loc)
+static bool is_shrouded(const display& disp, const map_location& loc)
 {
-	return disp.shrouded(loc) || disp.shrouded(gamemap::location(loc.x,loc.y+1));
+	return disp.shrouded(loc) || disp.shrouded(map_location(loc.x,loc.y+1));
 }
 
 map_labels::map_labels(const display& disp,
-					   const gamemap& map,
 					   const team* team) :
 		disp_(disp),
 		team_(team),
-		map_(map)
+		labels_(),
+		label_cache_()
 {
 }
 
 map_labels::map_labels(const display& disp,
 		       const config& cfg,
-		       const gamemap& map,
 		       const team* team,
 		       const variable_set *variables) :
 		disp_(disp),
 		team_(team),
-		map_(map)
+		labels_(),
+		label_cache_()
 {
 	read(cfg, variables);
 }
@@ -81,7 +78,7 @@ void map_labels::read(const config& cfg, const variable_set *variables)
 
 	const config::child_list& items = cfg.get_children("label");
 	for(config::child_list::const_iterator i = items.begin(); i != items.end(); ++i) {
-		const gamemap::location loc(**i, variables);
+		const map_location loc(**i, variables);
 		terrain_label* label = new terrain_label(*this, **i, variables);
 		add_label(loc, label);
 	}
@@ -94,18 +91,18 @@ size_t map_labels::get_max_chars()
 	return max_label_size;
 }
 
-const terrain_label* map_labels::get_label(const gamemap::location& loc, const std::string& team_name)
+const terrain_label* map_labels::get_label(const map_location& loc, const std::string& team_name)
 {
 	team_label_map::const_iterator label_map = labels_.find(team_name);
 	if (label_map != labels_.end()) {
-		map_labels::label_map::const_iterator itor = label_map->second.find(loc);;	
+		map_labels::label_map::const_iterator itor = label_map->second.find(loc);;
 		if (itor != label_map->second.end())
 			return itor->second;
 	}
 	return NULL;
 }
 
-const terrain_label* map_labels::get_label(const gamemap::location& loc)
+const terrain_label* map_labels::get_label(const map_location& loc)
 {
 	const terrain_label* res = get_label(loc, team_name());
 	// no such team label, we try global label, except if it's what we just did
@@ -141,10 +138,11 @@ void map_labels::set_team(const team* team)
 }
 
 
-const terrain_label* map_labels::set_label(const gamemap::location& loc,
+const terrain_label* map_labels::set_label(const map_location& loc,
 					   const std::string& text,
 					   const std::string team_name,
-					   const SDL_Color colour)
+					   const SDL_Color colour,
+					   const bool visible_in_fog)
 {
 	terrain_label* res = 0;
 	const team_label_map::const_iterator current_label_map = labels_.find(team_name);
@@ -157,7 +155,7 @@ const terrain_label* map_labels::set_label(const gamemap::location& loc,
 		if(text.empty())
 		{
 			const_cast<terrain_label*>(current_label->second)->set_text("");
-			res = new terrain_label("",team_name,loc,*this,colour);
+			res = new terrain_label("",team_name,loc,*this,colour,visible_in_fog);
 			delete current_label->second;
 			const_cast<label_map&>(current_label_map->second).erase(loc);
 
@@ -196,7 +194,8 @@ const terrain_label* map_labels::set_label(const gamemap::location& loc,
 				team_name,
 				loc,
 				*this,
-				colour);
+				colour,
+				visible_in_fog);
 		add_label(loc,label);
 
 		res = label;
@@ -210,7 +209,7 @@ const terrain_label* map_labels::set_label(const gamemap::location& loc,
 	return res;
 }
 
-void map_labels::add_label(const gamemap::location& loc,
+void map_labels::add_label(const map_location& loc,
 	const terrain_label* new_label)
 {
 	team_label_map::const_iterator labs = labels_.find(new_label->team_name());
@@ -222,7 +221,7 @@ void map_labels::add_label(const gamemap::location& loc,
 		assert(labs != labels_.end());
 	}
 
-	const_cast<label_map&>(labs->second).insert(std::pair<gamemap::location,const terrain_label*>(loc,new_label));
+	const_cast<label_map&>(labs->second).insert(std::pair<map_location,const terrain_label*>(loc,new_label));
 }
 
 void map_labels::clear(const std::string& team_name)
@@ -280,7 +279,7 @@ void map_labels::recalculate_labels()
 	}
 }
 
-bool map_labels::visible_global_label(const gamemap::location& loc) const
+bool map_labels::visible_global_label(const map_location& loc) const
 {
 	const team_label_map::const_iterator glabels = labels_.find(team_name());
 	return glabels == labels_.end()
@@ -302,12 +301,14 @@ void map_labels::recalculate_shroud()
 /// creating new label
 terrain_label::terrain_label(const std::string& text,
 							 const std::string& team_name,
-							 const gamemap::location& loc,
+							 const map_location& loc,
 							 const map_labels& parent,
-							 const SDL_Color colour)  :
+							 const SDL_Color colour,
+							 const bool visible_in_fog)  :
 		handle_(0),
 		text_(text),
 		team_name_(team_name),
+		visible_in_fog_(visible_in_fog),
 		colour_(colour),
 		parent_(&parent),
 		loc_(loc)
@@ -320,6 +321,7 @@ terrain_label::terrain_label(const map_labels& parent)  :
 		handle_(0),
 		text_(),
 		team_name_(),
+		visible_in_fog_(true),
 		colour_(),
 		parent_(&parent),
 		loc_()
@@ -332,7 +334,12 @@ terrain_label::terrain_label(const map_labels& parent,
 			     const config& cfg,
 			     const variable_set *variables) :
 		handle_(0),
-		parent_(&parent)
+		text_(),
+		team_name_(),
+		visible_in_fog_(true),
+		colour_(),
+		parent_(&parent),
+		loc_()
 {
 	read(cfg, variables);
 	check_text_length();
@@ -346,12 +353,13 @@ terrain_label::~terrain_label()
 
 void terrain_label::read(const config& cfg, const variable_set *variables)
 {
-	loc_ = gamemap::location(cfg, variables);
+	loc_ = map_location(cfg, variables);
 	SDL_Color colour = font::LABEL_COLOUR;
 	std::string tmp_colour = cfg["colour"];
 
 	text_      = cfg["text"];
 	team_name_ = cfg["team_name"];
+	visible_in_fog_ = utils::string_bool(cfg["visible_in_fog"],true);
 
 	if (variables)
 	{
@@ -383,7 +391,7 @@ void terrain_label::write(config& cfg) const
 	cfg["text"] = text();
 	cfg["team_name"] = (this->team_name());
 	cfg["colour"] = cfg_colour();
-
+	cfg["visible_in_fog"] = visible_in_fog() ? "yes" : "no";
 }
 
 const std::string& terrain_label::text() const
@@ -396,7 +404,12 @@ const std::string& terrain_label::team_name() const
 	return team_name_;
 }
 
-const gamemap::location& terrain_label::location() const
+bool terrain_label::visible_in_fog() const
+{
+	return visible_in_fog_;
+}
+
+const map_location& terrain_label::location() const
 {
 	return loc_;
 }
@@ -406,7 +419,7 @@ const SDL_Colour& terrain_label::colour() const
 	return colour_;
 }
 
-const std::string terrain_label::cfg_colour() const
+std::string terrain_label::cfg_colour() const
 {
 	std::stringstream buf;
 	const unsigned int red = static_cast<unsigned int>(colour_.r);
@@ -465,12 +478,14 @@ void terrain_label::calculate_shroud() const
 
 void terrain_label::draw()
 {
+	if (text_.empty())
+		return;
 	clear();
 	if (visible())
 	{
 
-		const gamemap::location loc_nextx(loc_.x+1,loc_.y);
-		const gamemap::location loc_nexty(loc_.x,loc_.y+1);
+		const map_location loc_nextx(loc_.x+1,loc_.y);
+		const map_location loc_nexty(loc_.x,loc_.y+1);
 		const int xloc = (parent_->disp().get_location_x(loc_) +
 				parent_->disp().get_location_x(loc_nextx)*2)/3;
 		const int yloc = parent_->disp().get_location_y(loc_nexty) - font::SIZE_NORMAL;
@@ -491,15 +506,22 @@ void terrain_label::draw()
 
 bool terrain_label::visible() const
 {
-	return  parent_->team_name() == team_name_
-			|| (team_name_.empty() && parent_->visible_global_label(loc_));
+	if (!visible_in_fog_)
+	{
+		if (parent_->disp().fogged(loc_))
+		{
+			return false;
+		}
+	}
+	return  (parent_->team_name() == team_name_
+			|| (team_name_.empty() && parent_->visible_global_label(loc_)));
 }
 
 void terrain_label::check_text_length()
 {
 	// The actual data is wide_strings so test in wide_string mode
-	// also cutting a wide_string at an arbritary place gives odd 
-	// problems. 
+	// also cutting a wide_string at an arbritary place gives odd
+	// problems.
 	utils::truncate_as_wstring(text_, parent_->get_max_chars());
 }
 
