@@ -1,6 +1,6 @@
-/* $Id: mapgen_dialog.cpp 24154 2008-02-29 18:17:30Z mordante $ */
+/* $Id: mapgen_dialog.cpp 33770 2009-03-17 19:41:19Z jhinrichs $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -19,16 +19,12 @@
 #include "mapgen_dialog.hpp"
 
 #include "display.hpp"
-#include "events.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
-#include "mapgen.hpp"
+#include "map.hpp"
 #include "marked-up_text.hpp"
 #include "show_dialog.hpp"
-#include "util.hpp"
-#include "video.hpp"
 
-#include "widgets/button.hpp"
 #include "widgets/slider.hpp"
 
 #define DBG_NG LOG_STREAM(debug, engine)
@@ -38,12 +34,22 @@ namespace {
 	const size_t max_coastal = 5;
 }
 
-default_map_generator::default_map_generator(const config* cfg)
-: width_(40), height_(40), island_size_(0), iterations_(1000), hill_size_(10), max_lakes_(20),
-  nvillages_(25), castle_size_(9), nplayers_(2), link_castles_(true)
+default_map_generator::default_map_generator(const config* cfg) :
+	default_width_(40),
+	default_height_(40),
+	width_(40),
+	height_(40),
+	island_size_(0),
+	iterations_(1000),
+	hill_size_(10),
+	max_lakes_(20),
+	nvillages_(25),
+	castle_size_(9),
+	nplayers_(2),
+	link_castles_(true),
+	cfg_(cfg ? *cfg : config())
 {
 	if(cfg != NULL) {
-		cfg_ = *cfg;
 
 		const int width = ::atoi((*cfg)["map_width"].c_str());
 		if(width > 0)
@@ -130,7 +136,7 @@ void default_map_generator::user_config(display& disp)
 
 	const int horz_margin = 15;
 	const int text_right = xpos + horz_margin +
-	        maximum<int>(maximum<int>(maximum<int>(maximum<int>(maximum<int>(maximum<int>(
+	        std::max<int>(std::max<int>(std::max<int>(std::max<int>(std::max<int>(std::max<int>(
 		         players_rect.w,width_rect.w),height_rect.w),iterations_rect.w),hillsize_rect.w),villages_rect.w),castlesize_rect.w);
 
 	players_rect.x = text_right - players_rect.w;
@@ -321,12 +327,22 @@ void default_map_generator::user_config(display& disp)
 
 std::string default_map_generator::name() const { return "default"; }
 
+std::string default_map_generator::config_name() const
+{
+	const config* c = cfg_.child("scenario");
+	if (c != NULL) {
+		return (*c)["name"];
+	} else {
+		return "";
+	}
+}
+
 std::string default_map_generator::create_map(const std::vector<std::string>& args)
 {
 	return generate_map(args);
 }
 
-std::string default_map_generator::generate_map(const std::vector<std::string>& /*args*/, std::map<gamemap::location,std::string>* labels)
+std::string default_map_generator::generate_map(const std::vector<std::string>& /*args*/, std::map<map_location,std::string>* labels)
 {
 	// the random generator thinks odd widths are nasty, so make them even
 	if (is_odd(width_))
@@ -351,19 +367,37 @@ std::string default_map_generator::generate_map(const std::vector<std::string>&
 		//the radius of the island should be up to twice the width of the map
 		const size_t island_radius = 40 + ((max_coastal - island_size_)*40)/max_coastal;
 		island_size = (island_radius*width_*2)/100;
-		island_off_center = minimum<size_t>(width_,height_);
+		island_off_center = std::min<size_t>(width_,height_);
 		DBG_NG << "calculated coastal params...\n";
 	}
 
 	// A map generator can fail so try a few times to get a map before aborting.
 	std::string map;
+	// Keep a copy of labels as it can be written to by the map generator func
+	std::map<map_location,std::string> labels_copy;
+	std::string error_message;
 	int tries = 10;
 	do {
-		map = default_generate_map(width_, height_, island_size, island_off_center,
-			iterations, hill_size_, max_lakes, (nvillages_ * width_ * height_) / 1000,
-			castle_size_, nplayers_, link_castles_, labels, cfg_);
-		--tries; 
+		if (labels) {
+				labels_copy = *labels;
+		}
+		try{
+			map = default_generate_map(width_, height_, island_size, island_off_center,
+				iterations, hill_size_, max_lakes, (nvillages_ * width_ * height_) / 1000,
+				castle_size_, nplayers_, link_castles_, labels, cfg_);
+			error_message = "";
+		}
+		catch (mapgen_exception& exc){
+			error_message = exc.msg;
+		}
+		--tries;
 	} while (tries && map.empty());
+	if (labels) {
+		labels->swap(labels_copy);
+	}
+
+	if (error_message != "")
+		throw mapgen_exception(error_message);
 
 	return map;
 }
@@ -380,12 +414,18 @@ config default_map_generator::create_scenario(const std::vector<std::string>& ar
 
 	DBG_NG << "got scenario data...\n";
 
-	std::map<gamemap::location,std::string> labels;
+	std::map<map_location,std::string> labels;
 	DBG_NG << "generating map...\n";
-	res["map_data"] = generate_map(args,&labels);
+	try{
+		res["map_data"] = generate_map(args,&labels);
+	}
+	catch (mapgen_exception exc){
+		res["map_data"] = "";
+		res["error_message"] = exc.msg;
+	}
 	DBG_NG << "done generating map..\n";
 
-	for(std::map<gamemap::location,std::string>::const_iterator i =
+	for(std::map<map_location,std::string>::const_iterator i =
 			labels.begin(); i != labels.end(); ++i) {
 
 		if(i->first.x >= 0 && i->first.y >= 0 &&
