@@ -1,4 +1,4 @@
-/* $Id: mouse_events.cpp 33687 2009-03-15 16:50:42Z mordante $ */
+/* $Id: mouse_events.cpp 36315 2009-06-21 10:47:04Z silene $ */
 /*
    Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playturn Copyright (C) 2003 by David White <dave@whitevine.net>
@@ -445,11 +445,14 @@ void mouse_handler::select_hex(const map_location& hex, const bool browse) {
 	if(hex.valid() && u != units_.end() && !u->second.get_hidden()) {
 		next_unit_ = u->first;
 
-		// if it's not the unit's turn, we reset its moves
-		unit_movement_resetter move_reset(u->second, u->second.side() != team_num_);
-		const bool teleport = u->second.get_ability_bool("teleport",u->first);
-		current_paths_ = paths(map_,units_,hex,teams_,
-						   false,teleport,viewing_team(),path_turns_);
+		{
+			// if it's not the unit's turn, we reset its moves
+			// and we restore them before the "select" event is raised
+			unit_movement_resetter move_reset(u->second, u->second.side() != team_num_);
+			bool teleport = u->second.get_ability_bool("teleport", u->first);
+			current_paths_ = paths(map_, units_, hex, teams_,
+				false, teleport, viewing_team(), path_turns_);
+		}
 		show_attack_options(u);
 		gui().highlight_reach(current_paths_);
 		// the highlight now comes from selection
@@ -654,8 +657,6 @@ bool mouse_handler::attack_enemy_(unit_map::iterator attacker, unit_map::iterato
 
 		gui().draw();
 
-		const bool defender_human = teams_[defender->second.side()-1].is_human();
-
 		recorder.add_attack(attacker_loc,defender_loc,att.attack_num,def.attack_num);
 
 		//MP_COUNTDOWN grant time bonus for attacking
@@ -667,12 +668,20 @@ bool mouse_handler::attack_enemy_(unit_map::iterator attacker, unit_map::iterato
 			//if the level ends due to a unit being killed, still see if
 			//either the attacker or defender should advance
 			dialogs::advance_unit(map_,units_,attacker_loc,gui());
-			dialogs::advance_unit(map_,units_,defender_loc,gui(),!defender_human);
+			unit_map::const_iterator defu = units_.find(defender_loc);
+			if (defu != units_.end()) {
+				bool defender_human = teams_[defu->second.side()-1].is_human();
+				dialogs::advance_unit(map_,units_,defender_loc,gui(),!defender_human);
+			}
 			throw;
 		}
 
 		dialogs::advance_unit(map_,units_,attacker_loc,gui());
-		dialogs::advance_unit(map_,units_,defender_loc,gui(),!defender_human);
+		unit_map::const_iterator defu = units_.find(defender_loc);
+		if (defu != units_.end()) {
+			bool defender_human = teams_[defu->second.side()-1].is_human();
+			dialogs::advance_unit(map_,units_,defender_loc,gui(),!defender_human);
+		}
 
 		check_victory(units_, teams_, gui());
 
