@@ -1,4 +1,4 @@
-/* $Id: multiplayer_connect.cpp 33040 2009-02-23 07:05:50Z dfranke $ */
+/* $Id: multiplayer_connect.cpp 37521 2009-08-06 13:32:09Z euschn $ */
 /*
    Copyright (C) 2007 - 2009
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -626,6 +626,13 @@ config connect::side::get_config() const
 		res.append(*(parent_->era_sides_[faction_]));
 	}
 	res.append(cfg_);
+
+	if (parent_->write_faction_name_) {
+		// res["name"] will be overwritten below by a 1.6.0 bug
+		// record it here in a key only used for MP UI communication
+		res["faction_name"] = res["name"];
+	}
+
 	if (cfg_.get_attribute("side").empty()
 			|| cfg_["side"] != lexical_cast<std::string>(index_ + 1))
 	{
@@ -987,6 +994,7 @@ connect::connect(game_display& disp, const config& game_config,
 	mp::ui(disp, _("Game Lobby: ") + params.name, game_config, c, gamelist),
 	local_only_(local_players_only),
 	level_(),
+	write_faction_name_(true),
 	state_(),
 	params_(params),
 	era_sides_(),
@@ -1020,6 +1028,9 @@ connect::connect(game_display& disp, const config& game_config,
 	if(get_result() == QUIT
 		|| get_result() == CREATE)
 		return;
+	if (level_["id"].empty()) {
+		throw config::error(_("The scenario is invalid because it has no id."));
+	}
 	lists_init();
 	if(sides_.empty()) {
 		throw config::error(_("The scenario is invalid because it has no sides."));
@@ -1115,6 +1126,9 @@ const game_state& connect::get_state()
 
 void connect::start_game()
 {
+	// Stop sending faction_name, not needed anymore
+	write_faction_name_ = false;
+
 	// Resolves the "random faction", "random gender" and "random message"
 	for (side_list::iterator itor = sides_.begin(); itor != sides_.end();
 			++itor) {
@@ -1596,6 +1610,7 @@ void connect::load_game()
 			level_.add_child("snapshot") = config();
 		} else {
 			level_.add_child("snapshot") = start_data;
+			level_.add_child("replay_start") = state_.starting_pos;
 		}
 		level_["random_seed"] = start_data["random_seed"];
 		level_["random_calls"] = start_data["random_calls"];
