@@ -1,4 +1,4 @@
-/* $Id: multiplayer_lobby.cpp 33741 2009-03-17 03:35:09Z soliton $ */
+/* $Id: multiplayer_lobby.cpp 36683 2009-07-07 13:36:09Z soliton $ */
 /*
    Copyright (C) 2007 - 2009
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -144,7 +144,7 @@ void gamebrowser::draw_row(const size_t index, const SDL_Rect& item_rect, ROW_TY
 			font_color = font::BAD_COLOUR;
 		}
 	}
-	if(!game.have_era) {
+	if(!game.have_era && font_color != font::BAD_COLOUR) {
 		font_color = font::DISABLED_COLOUR;
 		no_era_string = _(" (Unknown Era)");
 	}
