@@ -1,4 +1,4 @@
-/* $Id: multiplayer_ui.cpp 33296 2009-03-03 01:17:48Z soliton $ */
+/* $Id: multiplayer_ui.cpp 37533 2009-08-07 02:24:48Z soliton $ */
 /*
    Copyright (C) 2005 - 2009
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -32,6 +32,8 @@
 #define LOG_NW LOG_STREAM(info, network)
 #define ERR_NW LOG_STREAM(err, network)
 
+#define MAX_MESSAGES 256
+
 namespace {
 
 	class user_menu_style : public gui::menu::imgsel_style {
@@ -200,7 +202,7 @@ void chat::add_message(const time_t& time, const std::string& user,
 {
 	message_history_.push_back(msg(time, user, message));
 
-	while (message_history_.size() > 1024) {
+	while (message_history_.size() > MAX_MESSAGES) {
 		message_history_.pop_front();
 
 		if (last_update_ > 0)
@@ -231,11 +233,10 @@ void chat::update_textbox(gui::textbox& textbox)
 std::string chat::format_message(const msg& message)
 {
 	std::string msg_text = message.message;
-	if(message.user == "server") {
-		SDL_Color c;
-		int unused;
+	if(message.user == "server"
+	|| message.user.substr(0,29) == "whisper: server message from ") {
 		std::string::const_iterator after_markup =
-			font::parse_markup(message.message.begin(), message.message.end(), &unused, &c, &unused);
+			font::parse_markup(message.message.begin(), message.message.end(), NULL, NULL, NULL);
 
 		msg_text = std::string(after_markup,message.message.end());
 	}
@@ -251,12 +252,15 @@ std::string chat::format_message(const msg& message)
 SDL_Color chat::color_message(const msg& message) {
 	SDL_Color c = font::NORMAL_COLOUR;
 	// Normal users are not allowed to color their messages
-	if(message.user == "server") {
-		int unused;
-		font::parse_markup(message.message.begin(), message.message.end(), &unused, &c, &unused);
+	if(message.user == "server"
+	|| message.user.substr(0,29) == "whisper: server message from ") {
+		font::parse_markup(message.message.begin(), message.message.end(), NULL, &c, NULL);
+	// Highlight private messages too
+	} else if(message.user.substr(0,8) == "whisper:") {
+	    c = font::LABEL_COLOUR;
 	}
 	return c;
-};
+}
 
 ui::ui(game_display& disp, const std::string& title, const config& cfg, chat& c, config& gamelist) :
 	gui::widget(disp.video()),
