@@ -1,4 +1,4 @@
-/* $Id: multiplayer_wait.cpp 32674 2009-02-08 13:02:29Z cib $ */
+/* $Id: multiplayer_wait.cpp 36137 2009-06-11 12:48:09Z ivanovic $ */
 /*
    Copyright (C) 2007 - 2009
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -450,10 +450,17 @@ void wait::generate_menu()
 			continue;
 		}
 
-		std::string description = sd["description"];
+		// 1.6.0 incorrectly uses sd["description"] which is empty
+		// std::string description = sd["description"];
+		std::string description = sd["name"];
 		const std::string faction_id = sd["id"];
 
-		t_string side_name = sd["name"];
+		// 1.6.0 incorrectly uses sd["name"] which is the player's name
+		// t_string side_name = sd["name"];
+
+		// 1.6.0 host will not send this to us, so the column will be empty
+		t_string side_name = sd["faction_name"];
+		
 		std::string leader_type = sd["type"];
 		std::string gender_id = sd["gender"];
 
@@ -526,7 +533,7 @@ void wait::generate_menu()
 		str << COLUMN_SEPARATOR;
 		// Don't show gold for saved games
 		if(sd["allow_changes"] == "yes")
-			str << sd["gold"] << ' ' << _("Gold") << COLUMN_SEPARATOR;
+			str << sd["gold"] << ' ' << _n("multiplayer_starting_gold^Gold", "multiplayer_starting_gold^Gold", lexical_cast_default<int>(sd["gold"], 0)) << COLUMN_SEPARATOR;
 
 		int income_amt = lexical_cast_default<int>(sd["income"], 0);
 		if(income_amt != 0){
