@@ -1,6 +1,6 @@
-/* $Id: network.cpp 27539 2008-06-27 19:58:18Z suokko $ */
+/* $Id: network.cpp 37795 2009-08-15 15:57:24Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,35 +12,28 @@
    See the COPYING file for more details.
 */
 
-//! @file network.cpp
-//! Networking
+/**
+ * @file network.cpp
+ * Networking
+ */
 
 #include "global.hpp"
 
-#include "serialization/binary_wml.hpp"
-#include "config.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
-#include "network.hpp"
 #include "network_worker.hpp"
 #include "thread.hpp"
 
 #include "filesystem.hpp"
 
-#include "SDL_net.h"
 
-#include <algorithm>
-#include <cassert>
 #include <cerrno>
 #include <queue>
-#include <iostream>
 #include <iomanip>
 #include <set>
-#include <vector>
-#include <ctime>
+#include <cstring>
 
 #include <signal.h>
-#include <string.h>
 #if defined(_WIN32) || defined(__WIN32__) || defined (WIN32)
 #undef INADDR_ANY
 #undef INADDR_BROADCAST
@@ -92,7 +85,7 @@ connection_map connections;
 
 network::connection connection_id = 1;
 
-//! Stores the time of the last server ping we received.
+/** Stores the time of the last server ping we received. */
 time_t last_ping, last_ping_check = 0;
 
 } // end anon namespace
@@ -146,10 +139,12 @@ static void check_error()
 	}
 }
 
-//! Check whether too much time since the last server ping has passed and we
-//! timed out. If the last check is too long ago reset the last_ping to 'now'.
-//! This happens when we "freeze" the client one way or another or we just
-//! didn't try to receive data.
+/**
+ * Check whether too much time since the last server ping has passed and we
+ * timed out. If the last check is too long ago reset the last_ping to 'now'.
+ * This happens when we "freeze" the client one way or another or we just
+ * didn't try to receive data.
+ */
 static void check_timeout()
 {
 	if (network::nconnections() == 0) {
@@ -164,14 +159,13 @@ static void check_timeout()
 	if (last_ping_check + 10 <= now) last_ping = now;
 	if (static_cast<time_t>(last_ping + network::ping_timeout) <= now) {
 
-		int timeout = now - last_ping;
+		time_t timeout = now - last_ping;
 		ERR_NW << "No server ping since " << timeout
 				<< " seconds. Connection timed out.\n";
 		utils::string_map symbols;
 		symbols["timeout"] = lexical_cast<std::string>(timeout);
-		throw network::error(vngettext("No server ping since $timeout second. "
-				"Connection timed out.", "No server ping since $timeout seconds. "
-				"Connection timed out.", timeout, symbols));
+		throw network::error(std::string("No server ping since " + lexical_cast<std::string>(timeout) + " second. "
+				"Connection timed out."));
 	}
 	last_ping_check = now;
 }
@@ -186,7 +180,12 @@ sockets_list sockets;
 
 
 struct partial_buffer {
-	partial_buffer() : upto(0) {}
+	partial_buffer() :
+		buf(),
+		upto(0)
+	{
+	}
+
 	std::vector<char> buf;
 	size_t upto;
 };
@@ -202,9 +201,11 @@ network_worker_pool::manager* worker_pool_man = NULL;
 
 namespace network {
 
-//! Amount of seconds after the last server ping when we assume to have timed out.
-//! When set to '0' ping timeout isn't checked.
-//! Gets set in preferences::manager according to the preferences file.
+/**
+ * Amount of seconds after the last server ping when we assume to have timed out.
+ * When set to '0' ping timeout isn't checked.
+ * Gets set in preferences::manager according to the preferences file.
+ */
 unsigned int ping_timeout = 0;
 
 connection_stats::connection_stats(int sent, int received, int connected_at)
@@ -236,6 +237,7 @@ pending_statistics get_pending_stats()
 
 manager::manager(size_t min_threads, size_t max_threads) : free_(true)
 {
+	DBG_NW << "NETWORK MANAGER CALLED!\n";
 	// If the network is already being managed
 	if(socket_set) {
 		free_ = false;
@@ -276,14 +278,15 @@ void set_raw_data_only()
 	network_worker_pool::set_raw_data_only();
 }
 
-server_manager::server_manager(int port, CREATE_SERVER create_server) : free_(false)
+server_manager::server_manager(int port, CREATE_SERVER create_server) : free_(false), connection_(0)
 {
 	if(create_server != NO_SERVER && !server_socket) {
 		try {
-			server_socket = get_socket(connect("",port));
-		} catch(network::error& e) {
+			connection_ = connect("",port);
+			server_socket = get_socket(connection_);
+		} catch(network::error&) {
 			if(create_server == MUST_CREATE_SERVER) {
-				throw e;
+				throw;
 			} else {
 				return;
 			}
@@ -303,6 +306,7 @@ void server_manager::stop()
 {
 	if(free_) {
 		SDLNet_TCP_Close(server_socket);
+		remove_connection(connection_);
 		server_socket = 0;
 		free_ = false;
 	}
@@ -404,12 +408,14 @@ void connect_operation::run()
 #endif
 	if(fcntl((reinterpret_cast<_TCPsocket*>(sock))->channel, F_SETFL, flags) == -1) {
 		error_ = ("Could not make socket non-blocking: " + std::string(strerror(errno))).c_str();
+		SDLNet_TCP_Close(sock);
 		return;
 	}
 #else
 	int on = 1;
 	if(setsockopt(((_TCPsocket*)sock)->channel, SOL_SOCKET, SO_NONBLOCK, &on, sizeof(int)) < 0) {
 		error_ = ("Could not make socket non-blocking: " + std::string(strerror(errno))).c_str();
+		SDLNet_TCP_Close(sock);
 		return;
 	}
 #endif
@@ -422,8 +428,8 @@ void connect_operation::run()
 	}
 
 	// Send data telling the remote host that this is a new connection
-	char buf[4];
-	SDLNet_Write32(0,buf);
+	char buf[4] ALIGN_4;
+	SDLNet_Write32(0, reinterpret_cast<void*>(buf));
 	const int nbytes = SDLNet_TCP_Send(sock,buf,4);
 	if(nbytes != 4) {
 		SDLNet_TCP_Close(sock);
@@ -455,7 +461,7 @@ void connect_operation::run()
 
 	sockets.push_back(connect_);
 
-	while(!notify_finished());
+	while(!notify_finished()) {};
 }
 
 
@@ -472,14 +478,14 @@ connection connect(const std::string& host, int port)
 
 connection connect(const std::string& host, int port, threading::waiter& waiter)
 {
-	const threading::async_operation_holder<connect_operation> op(new connect_operation(host,port));
-	const connect_operation::RESULT res = op.operation().execute(waiter);
+	const threading::async_operation_ptr op(new connect_operation(host,port));
+	const connect_operation::RESULT res = op->execute(op, waiter);
 	if(res == connect_operation::ABORTED) {
 		return 0;
 	}
 
-	op.operation().check_error();
-	return op.operation().result();
+	static_cast<connect_operation*>(op.get())->check_error();
+	return static_cast<connect_operation*>(op.get())->result();
 }
 
 connection accept_connection()
@@ -492,8 +498,10 @@ connection accept_connection()
 	// The initial handshake is a 4 byte value, which is 0 for a new connection,
 	// or the handle of the connection if it's trying to recover a lost connection.
 
-	//! A list of all the sockets which have connected,
-	//! but haven't had their initial handshake received.
+	/**
+	 * A list of all the sockets which have connected,
+	 * but haven't had their initial handshake received.
+	 */
 	static std::vector<TCPsocket> pending_sockets;
 	static SDLNet_SocketSet pending_socket_set = 0;
 
@@ -502,11 +510,17 @@ connection accept_connection()
 		DBG_NW << "received connection. Pending handshake...\n";
 		pending_sockets.push_back(sock);
 		if(pending_socket_set == 0) {
-			pending_socket_set = SDLNet_AllocSocketSet(512);
+			pending_socket_set = SDLNet_AllocSocketSet(32);
 		}
 
 		if(pending_socket_set != 0) {
-			SDLNet_TCP_AddSocket(pending_socket_set,sock);
+			int res = SDLNet_TCP_AddSocket(pending_socket_set,sock);
+
+			if (res == -1)
+			{
+				ERR_NW << "Pending socket set is full! Disconnecting " << sock << " connection\n";
+				SDLNet_TCP_Close(sock);
+			}
 		}
 	}
 
@@ -528,7 +542,7 @@ connection accept_connection()
 
 		// Receive the 4 bytes telling us if they're a new connection
 		// or trying to recover a connection
-		char buf[4];
+		char buf[4] ALIGN_4;
 
 		const TCPsocket sock = *i;
 		SDLNet_TCP_DelSocket(pending_socket_set,sock);
@@ -543,7 +557,7 @@ connection accept_connection()
 			return 0;
 		}
 
-		const int handle = SDLNet_Read32(buf);
+		const int handle = SDLNet_Read32(reinterpret_cast<void*>(buf));
 
 		DBG_NW << "received handshake from client: '" << handle << "'\n";
 
@@ -558,7 +572,7 @@ connection accept_connection()
 		const connection connect = create_connection(sock,"",0);
 
 		// Send back their connection number
-		SDLNet_Write32(connect,buf);
+		SDLNet_Write32(connect, reinterpret_cast<void*>(buf));
 		const int nbytes = SDLNet_TCP_Send(sock,buf,4);
 		if(nbytes != 4) {
 			SDLNet_TCP_DelSocket(socket_set,sock);
@@ -575,12 +589,11 @@ connection accept_connection()
 	return 0;
 }
 
-bool disconnect(connection s, bool force)
+bool disconnect(connection s)
 {
 	if(s == 0) {
 		while(sockets.empty() == false) {
 			assert(sockets.back() != 0);
-//			disconnect(sockets.back(), true);
 			while(disconnect(sockets.back()) == false) {
 				SDL_Delay(1);
 			}
@@ -591,7 +604,11 @@ bool disconnect(connection s, bool force)
 
 	const connection_map::iterator info = connections.find(s);
 	if(info != connections.end()) {
-		if (!network_worker_pool::close_socket(info->second.sock, force)) {
+		if (info->second.sock == server_socket)
+		{
+			return true;
+		}
+		if (!network_worker_pool::close_socket(info->second.sock)) {
 			return false;
 		}
 	}
@@ -628,19 +645,11 @@ void queue_disconnect(network::connection sock)
 	disconnection_queue.push_back(sock);
 }
 
-connection receive_data(config& cfg, connection connection_num, unsigned int timeout
-#ifdef BANDWIDTH_MONITOR
-		, bandwidth_in_ptr* bandwidth_in
-#endif
-		)
+connection receive_data(config& cfg, connection connection_num, unsigned int timeout, bandwidth_in_ptr* bandwidth_in)
 {
 	unsigned int start_ticks = SDL_GetTicks();
 	while(true) {
-		const connection res = receive_data(cfg,connection_num
-#ifdef BANDWIDTH_MONITOR
-		,(bool*)0 , bandwidth_in
-#endif
-		);
+		const connection res = receive_data(cfg,connection_num,(bool*)0 , bandwidth_in);
 		if(res != 0) {
 			return res;
 		}
@@ -658,11 +667,7 @@ connection receive_data(config& cfg, connection connection_num, unsigned int tim
 	return 0;
 }
 
-connection receive_data(config& cfg, connection connection_num, bool* gzipped
-#ifdef BANDWIDTH_MONITOR
-		, bandwidth_in_ptr* bandwidth_in
-#endif
-		)
+connection receive_data(config& cfg, connection connection_num, bool* gzipped, bandwidth_in_ptr* bandwidth_in)
 {
 	if(!socket_set) {
 		return 0;
@@ -694,13 +699,13 @@ connection receive_data(config& cfg, connection connection_num, bool* gzipped
 			// See if this socket is still waiting for it to be assigned its remote handle.
 			// If it is, then the first 4 bytes must be the remote handle.
 			if(is_pending_remote_handle(*i)) {
-				char buf[4];
+				char buf[4] ALIGN_4;
 				int len = SDLNet_TCP_Recv(sock,buf,4);
 				if(len != 4) {
 					throw error("Remote host disconnected",*i);
 				}
 
-				const int remote_handle = SDLNet_Read32(buf);
+				const int remote_handle = SDLNet_Read32(reinterpret_cast<void*>(buf));
 				set_remote_handle(*i,remote_handle);
 
 				continue;
@@ -717,18 +722,12 @@ connection receive_data(config& cfg, connection connection_num, bool* gzipped
 
 	TCPsocket sock = connection_num == 0 ? 0 : get_socket(connection_num);
 	TCPsocket s = sock;
-#ifdef BANDWIDTH_MONITOR
 	bandwidth_in_ptr temp;
 	if (!bandwidth_in)
 	{
 		bandwidth_in = &temp;
 	}
-#endif
-	sock = network_worker_pool::get_received_data(sock,cfg, gzipped
-#ifdef BANDWIDTH_MONITOR
-			, *bandwidth_in
-#endif
-			);
+	sock = network_worker_pool::get_received_data(sock,cfg, gzipped, *bandwidth_in);
 	if (sock == NULL) {
 		if (!is_server() && last_ping != 0 && ping_timeout != 0)
 		{
@@ -744,7 +743,13 @@ connection receive_data(config& cfg, connection connection_num, bool* gzipped
 		return 0;
 	}
 
-	SDLNet_TCP_AddSocket(socket_set,sock);
+	int set_res = SDLNet_TCP_AddSocket(socket_set,sock);
+	if (set_res == -1)
+	{
+		ERR_NW << "Socket set is full! Disconnecting " << sock << " connection\n";
+		SDLNet_TCP_Close(sock);
+		return 0;
+	}
 
 	connection result = 0;
 	for(connection_map::const_iterator j = connections.begin(); j != connections.end(); ++j) {
@@ -754,7 +759,7 @@ connection receive_data(config& cfg, connection connection_num, bool* gzipped
 		}
 	}
 	if(!cfg.empty()) {
-		DBG_NW << "RECEIVED from: " << result << ": " << cfg; 
+		DBG_NW << "RECEIVED from: " << result << ": " << cfg;
 	}
 
 	assert(result != 0);
@@ -767,16 +772,12 @@ connection receive_data(config& cfg, connection connection_num, bool* gzipped
 			last_ping = now;
 		} else if (last_ping != 0) {
 			last_ping = now;
-		}			
+		}
 	}
 	return result;
 }
 
-connection receive_data(std::vector<char>& buf
-#ifdef BANDWIDTH_MONITOR
-		, bandwidth_in_ptr* bandwidth_in
-#endif
-		)
+connection receive_data(std::vector<char>& buf, bandwidth_in_ptr* bandwidth_in)
 {
 	if(!socket_set) {
 		return 0;
@@ -808,13 +809,13 @@ connection receive_data(std::vector<char>& buf
 			// See if this socket is still waiting for it to be assigned its remote handle.
 			// If it is, then the first 4 bytes must be the remote handle.
 			if(is_pending_remote_handle(*i)) {
-				char buf[4];
+				char buf[4] ALIGN_4;
 				int len = SDLNet_TCP_Recv(sock,buf,4);
 				if(len != 4) {
 					throw error("Remote host disconnected",*i);
 				}
 
-				const int remote_handle = SDLNet_Read32(buf);
+				const int remote_handle = SDLNet_Read32(reinterpret_cast<void*>(buf));
 				set_remote_handle(*i,remote_handle);
 
 				continue;
@@ -834,20 +835,24 @@ connection receive_data(std::vector<char>& buf
 		return 0;
 	}
 
-#ifdef BANDWIDTH_MONITOR
 	{
 		bandwidth_in_ptr temp;
 		if (!bandwidth_in)
 		{
 			bandwidth_in = &temp;
 		}
-		const int headers = 5;
+		const int headers = 4;
 		bandwidth_in->reset(new network::bandwidth_in(buf.size() + headers));
 	}
-#endif
 
-	SDLNet_TCP_AddSocket(socket_set,sock);
+	int set_res = SDLNet_TCP_AddSocket(socket_set,sock);
 
+	if (set_res == -1)
+	{
+		ERR_NW << "Socket set is full! Disconnecting " << sock << " connection\n";
+		SDLNet_TCP_Close(sock);
+		return 0;
+	}
 	connection result = 0;
 	for(connection_map::const_iterator j = connections.begin(); j != connections.end(); ++j) {
 		if(j->second.sock == sock) {
@@ -860,7 +865,6 @@ connection receive_data(std::vector<char>& buf
 	waiting_sockets.insert(result);
 	return result;
 }
-#ifdef BANDWIDTH_MONITOR
 struct bandwidth_stats {
 	int out_packets;
 	int out_bytes;
@@ -886,7 +890,7 @@ hour_stats_vector hour_stats(24);
 
 
 
-static bandwidth_map::iterator add_bandwidth_entry(const std::string packet_type)
+static bandwidth_map::iterator add_bandwidth_entry(const std::string& packet_type)
 {
 	time_t now = time(0);
 	struct tm * timeinfo = localtime(&now);
@@ -894,7 +898,7 @@ static bandwidth_map::iterator add_bandwidth_entry(const std::string packet_type
 	int day = timeinfo->tm_mday;
 	assert(hour < 24 && hour >= 0);
 	std::pair<bandwidth_map::iterator,bool> insertion = hour_stats[hour].insert(std::make_pair(packet_type, bandwidth_stats()));
-	bandwidth_map::iterator& inserted = insertion.first;
+	bandwidth_map::iterator inserted = insertion.first;
 	if (!insertion.second && day != inserted->second.day)
 	{
 		// clear previuos day stats
@@ -903,6 +907,7 @@ static bandwidth_map::iterator add_bandwidth_entry(const std::string packet_type
 		insertion = hour_stats[hour].insert(std::make_pair(packet_type, bandwidth_stats()));
 		inserted = insertion.first;
 	}
+
 	inserted->second.day = day;
 	return inserted;
 }
@@ -915,7 +920,7 @@ struct bandwidth_stats_output {
 	{}
 	void operator()(const bandwidth_map::value_type& stats)
 	{
-		// name 
+		// name
 		ss_	<< " " << std::setw(bandwidth_stats::type_width) <<  stats.first << "| "
 			<< std::setw(bandwidth_stats::packet_width)<< stats.second.out_packets << "| "
 			<< std::setw(bandwidth_stats::bytes_width) << stats.second.out_bytes/1024 << "| "
@@ -957,9 +962,9 @@ std::string get_bandwidth_stats(int hour)
 	assert(hour < 24 && hour >= 0);
 	std::stringstream ss;
 
-	ss << "Hour stat starting from " << hour << "\n " << std::left << std::setw(bandwidth_stats::type_width) <<  "Type of packet" << "| " 
+	ss << "Hour stat starting from " << hour << "\n " << std::left << std::setw(bandwidth_stats::type_width) <<  "Type of packet" << "| "
 		<< std::setw(bandwidth_stats::packet_width)<< "out #"  << "| "
-		<< std::setw(bandwidth_stats::bytes_width) << "out kb" << "| "
+		<< std::setw(bandwidth_stats::bytes_width) << "out kb" << "| " /* Are these bytes or bits? base10 or base2? */
 		<< std::setw(bandwidth_stats::packet_width)<< "in #"  << "| "
 		<< std::setw(bandwidth_stats::bytes_width) << "in kb" << "\n";
 
@@ -970,14 +975,14 @@ std::string get_bandwidth_stats(int hour)
 	return ss.str();
 }
 
-void add_bandwidth_out(const std::string packet_type, size_t len)
+void add_bandwidth_out(const std::string& packet_type, size_t len)
 {
 	bandwidth_map::iterator itor = add_bandwidth_entry(packet_type);
 	itor->second.out_bytes += len;
 	++(itor->second.out_packets);
 }
 
-void add_bandwidth_in(const std::string packet_type, size_t len)
+void add_bandwidth_in(const std::string& packet_type, size_t len)
 {
 	bandwidth_map::iterator itor = add_bandwidth_entry(packet_type);
 	itor->second.in_bytes += len;
@@ -989,9 +994,7 @@ void add_bandwidth_in(const std::string packet_type, size_t len)
 		add_bandwidth_in(type_, len_);
 	}
 
-#endif
-void send_file(const std::string& filename, connection connection_num, const std::string packet_type
-		)
+void send_file(const std::string& filename, connection connection_num, const std::string& packet_type)
 {
 	assert(connection_num > 0);
 	if(bad_sockets.count(connection_num) || bad_sockets.count(0)) {
@@ -1005,62 +1008,53 @@ void send_file(const std::string& filename, connection connection_num, const std
 		return;
 	}
 
-#ifdef BANDWIDTH_MONITOR
-	const int packet_headers = 5;
+	const int packet_headers = 4;
 	add_bandwidth_out(packet_type, file_size(filename) + packet_headers);
-#endif
 	network_worker_pool::queue_file(info->second.sock, filename);
-	
+
 }
 
-//! @todo Note the gzipped parameter should be removed later, we want to send
-//! all data gzipped. This can be done once the campaign server is also updated
-//! to work with gzipped data.
-void send_data(const config& cfg, connection connection_num, const bool gzipped, const std::string packet_type
-		)
+/**
+ * @todo Note the gzipped parameter should be removed later, we want to send
+ * all data gzipped. This can be done once the campaign server is also updated
+ * to work with gzipped data.
+ */
+size_t send_data(const config& cfg, connection connection_num, const bool gzipped, const std::string& packet_type)
 {
 	DBG_NW << "in send_data()...\n";
-	
+
 	if(cfg.empty()) {
-		return;
+		return 0;
 	}
 
 	if(bad_sockets.count(connection_num) || bad_sockets.count(0)) {
-		return;
+		return 0;
 	}
 
 //	log_scope2(network, "sending data");
 	if(!connection_num) {
 		DBG_NW << "sockets: " << sockets.size() << "\n";
+		size_t size = 0;
 		for(sockets_list::const_iterator i = sockets.begin();
 		    i != sockets.end(); ++i) {
 			DBG_NW << "server socket: " << server_socket << "\ncurrent socket: " << *i << "\n";
-			send_data(cfg,*i, gzipped
-#ifdef BANDWIDTH_MONITOR
-					, packet_type
-#endif
-					);
+			size = send_data(cfg,*i, gzipped, packet_type);
 		}
-		return;
+		return size;
 	}
 
 	const connection_map::iterator info = connections.find(connection_num);
 	if (info == connections.end()) {
 		ERR_NW << "Error: socket: " << connection_num
 			<< "\tnot found in connection_map. Not sending...\n";
-		return;
+		return 0;
 	}
 
 	LOG_NW << "SENDING to: " << connection_num << ": " << cfg;
-	network_worker_pool::queue_data(info->second.sock, cfg, gzipped
-#ifdef BANDWIDTH_MONITOR
-			, packet_type
-#endif 
-			);
+	return network_worker_pool::queue_data(info->second.sock, cfg, gzipped, packet_type);
 }
 
-void send_raw_data(const char* buf, int len, connection connection_num, const std::string packet_type
-		)
+void send_raw_data(const char* buf, int len, connection connection_num, const std::string& packet_type)
 {
 	if(len == 0) {
 		return;
@@ -1073,8 +1067,7 @@ void send_raw_data(const char* buf, int len, connection connection_num, const st
 	if(!connection_num) {
 		for(sockets_list::const_iterator i = sockets.begin();
 		    i != sockets.end(); ++i) {
-			send_raw_data(buf, len, connection_num, packet_type
-					);
+			send_raw_data(buf, len, *i, packet_type);
 		}
 		return;
 	}
@@ -1085,10 +1078,8 @@ void send_raw_data(const char* buf, int len, connection connection_num, const st
 			<< "\tnot found in connection_map. Not sending...\n";
 		return;
 	}
-#ifdef BANDWIDTH_MONITOR
-	const int packet_headers = 5;
+	const int packet_headers = 4;
 	add_bandwidth_out(packet_type, len + packet_headers);
-#endif
 
 	network_worker_pool::queue_raw_data(info->second.sock, buf, len);
 }
@@ -1098,20 +1089,15 @@ void process_send_queue(connection, size_t)
 	check_error();
 }
 
-//! @todo Note the gzipped parameter should be removed later.
-void send_data_all_except(const config& cfg, connection connection_num, const bool gzipped, const std::string packet_type
-		)
+/** @todo Note the gzipped parameter should be removed later. */
+void send_data_all_except(const config& cfg, connection connection_num, const bool gzipped, const std::string& packet_type)
 {
 	for(sockets_list::const_iterator i = sockets.begin(); i != sockets.end(); ++i) {
 		if(*i == connection_num) {
 			continue;
 		}
 
-		send_data(cfg,*i, gzipped
-#ifdef BANDWIDTH_MONITOR
-				, packet_type
-#endif
-				);
+		send_data(cfg,*i, gzipped, packet_type);
 	}
 }
 
