@@ -1,4 +1,4 @@
-/* $Id: network_worker.cpp 33687 2009-03-15 16:50:42Z mordante $ */
+/* $Id: network_worker.cpp 37795 2009-08-15 15:57:24Z mordante $ */
 /*
    Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -234,9 +234,11 @@ bool receive_with_timeout(TCPsocket s, char* buf, size_t nbytes,
 			if(errno == EAGAIN)
 #elif defined(EWOULDBLOCK)
 			if(errno == EWOULDBLOCK)
+#elif defined(_WIN32) && defined(WSAEWOULDBLOCK)
+			if(WSAGetLastError() == WSAEWOULDBLOCK)
 #else
-			// Ignore the error.
-			if(true)
+			// assume non-recoverable error.
+			if(false)
 #endif
 			{
 #ifdef USE_POLL
@@ -416,8 +418,10 @@ static SOCKET_STATE send_buffer(TCPsocket sock, std::vector<char>& buf, int in_s
 			}
 			return SOCKET_READY;
 		}
-#if defined(EAGAIN) && !defined(__BEOS__) && !defined(_WIN32)
-	        if(errno == EAGAIN)
+#if defined(_WIN32)
+		if(WSAGetLastError() == WSAEWOULDBLOCK)
+#elif defined(EAGAIN) && !defined(__BEOS__)
+		if(errno == EAGAIN)
 #elif defined(EWOULDBLOCK)
 		if(errno == EWOULDBLOCK)
 #endif
@@ -614,7 +618,7 @@ static SOCKET_STATE receive_buf(TCPsocket sock, std::vector<char>& buf)
 		return SOCKET_ERRORED;
 	}
 
-	const int len = SDLNet_Read32(num_buf);
+	const int len = SDLNet_Read32(reinterpret_cast<void*>(num_buf));
 
 	if(len < 1 || len > 100000000) {
 		return SOCKET_ERRORED;
