@@ -1,6 +1,6 @@
-/* $Id: network_worker.hpp 28347 2008-08-06 15:28:35Z suokko $ */
+/* $Id: network_worker.hpp 33687 2009-03-15 16:50:42Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -15,20 +15,27 @@
 #ifndef NETWORK_WORKER_HPP_INCLUDED
 #define NETWORK_WORKER_HPP_INCLUDED
 
-// Check if sendfile function is implemented by glibc
-// It is linux only extension
-#if defined(__linux__) && defined(__GNUC_PREREQ)
-#if __GNUC_PREREQ(2,1)
-#define USE_SENDFILE
-#endif
-#endif
-
 #include <map>
 #include <vector>
 #include "config.hpp"
 #include "network.hpp"
 #include "SDL_net.h"
 
+/**
+ * Aligns a variable on a 4 byte boundry.
+ *
+ * The address needs to be aligned on a Sparc system, if it's not aligned the
+ * SDLNet_Read32 call will cause a SIGBUS and the server will be terminated [1].
+ * Best use this alignment for all buffers used in for the SDL_Net calls.
+ *
+ * [1] http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=426318
+ */
+#ifdef __GNUC__
+#define ALIGN_4 __attribute__ ((aligned (4)))
+#else
+#define ALIGN_4
+#endif
+
 namespace network_worker_pool
 {
 
@@ -47,27 +54,21 @@ private:
 network::pending_statistics get_pending_stats();
 
 void set_raw_data_only();
+void set_use_system_sendfile(bool);
 
-//! Function to asynchronously received data to the given socket.
+/** Function to asynchronously received data to the given socket. */
 void receive_data(TCPsocket sock);
 
-TCPsocket get_received_data(TCPsocket sock, config& cfg, bool* gzipped
-#ifdef BANDWIDTH_MONITOR
-		, network::bandwidth_in_ptr& 
-#endif
-		);
+TCPsocket get_received_data(TCPsocket sock, config& cfg, bool* gzipped, network::bandwidth_in_ptr&);
+
 TCPsocket get_received_data(std::vector<char>& buf);
 
 void queue_file(TCPsocket sock, const std::string&);
 
 void queue_raw_data(TCPsocket sock, const char* buf, int len);
-size_t queue_data(TCPsocket sock, const config& buf, const bool gzipped
-#ifdef BANDWIDTH_MONITOR
-		, const std::string& packet_type
-#endif
-		);
+size_t queue_data(TCPsocket sock, const config& buf, const bool gzipped, const std::string& packet_type);
 bool is_locked(const TCPsocket sock);
-bool close_socket(TCPsocket sock, bool force=false);
+bool close_socket(TCPsocket sock);
 TCPsocket detect_error();
 
 std::pair<network::statistics,network::statistics> get_current_transfer_stats(TCPsocket sock);
