@@ -1,7 +1,7 @@
-/* $Id: playcampaign.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: playcampaign.cpp 32891 2009-02-17 22:53:25Z jhinrichs $ */
 /*
    Copyright (C) 2003-2005 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2008 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 - 2009 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -13,30 +13,25 @@
    See the COPYING file for more details.
 */
 
-//! @file playcampaign.cpp 
-//! Controls setup, play, (auto)save and replay of campaigns.
+/**
+ *  @file playcampaign.cpp
+ *  Controls setup, play, (auto)save and replay of campaigns.
+ */
 
 #include "global.hpp"
 
 #include "playcampaign.hpp"
-#include "config.hpp"
-#include "filesystem.hpp"
-#include "game_preferences.hpp"
-#include "gamestatus.hpp"
 #include "map_create.hpp"
 #include "playmp_controller.hpp"
-#include "playsingle_controller.hpp"
-#include "replay.hpp"
 #include "replay_controller.hpp"
 #include "log.hpp"
+#include "map_exception.hpp"
 #include "dialogs.hpp"
 #include "gettext.hpp"
-#include "game_errors.hpp"
 #include "sound.hpp"
 #include "wml_exception.hpp"
+#include "formula_string_utils.hpp"
 
-#include <cassert>
-#include <map>
 
 #define LOG_G LOG_STREAM(info, general)
 #define LOG_NG LOG_STREAM(info, engine)
@@ -51,7 +46,7 @@ struct player_controller
 		{}
 
 	player_controller(const std::string& controller, const std::string& description) :
-		controller(controller), 
+		controller(controller),
 		description(description)
 		{}
 
@@ -61,10 +56,10 @@ struct player_controller
 
 typedef std::map<std::string, player_controller> controller_map;
 
-} // end anon namespace 
+} // end anon namespace
 
 void play_replay(display& disp, game_state& gamestate, const config& game_config,
-		const game_data& units_data, CVideo& video)
+		CVideo& video)
 {
 	std::string type = gamestate.campaign_type;
 	if(type.empty())
@@ -78,6 +73,7 @@ void play_replay(display& disp, game_state& gamestate, const config& game_config
 	if (gamestate.starting_pos.empty()){
 		// Backwards compatibility code for 1.2 and 1.2.1
 		scenario = game_config.find_child(type,"id",gamestate.scenario);
+		assert(scenario != NULL);
 		gamestate.starting_pos = *scenario;
 	}
 	recorder.set_save_info(gamestate);
@@ -91,7 +87,7 @@ void play_replay(display& disp, game_state& gamestate, const config& game_config
 		//if (gamestate.abbrev.empty())
 		//	gamestate.abbrev = (*scenario)["abbrev"];
 
-		play_replay_level(units_data,game_config,scenario,video,gamestate);
+		play_replay_level(game_config,scenario,video,gamestate);
 
 		gamestate.snapshot = config();
 		recorder.clear();
@@ -101,7 +97,7 @@ void play_replay(display& disp, game_state& gamestate, const config& game_config
 		gui::show_error_message(disp, _("The game could not be loaded: ") + e.message);
 	} catch(game::game_error& e) {
 		gui::show_error_message(disp, _("Error while playing the game: ") + e.message);
-	} catch(gamemap::incorrect_format_exception& e) {
+	} catch(incorrect_map_format_exception& e) {
 		gui::show_error_message(disp, std::string(_("The game map could not be loaded: ")) + e.msg_);
 	} catch(twml_exception& e) {
 		e.show(disp);
@@ -121,17 +117,17 @@ static void clean_saves(const std::string &label)
 	}
 }
 
-LEVEL_RESULT playsingle_scenario(const game_data& gameinfo, const config& game_config,
+static LEVEL_RESULT playsingle_scenario(const config& game_config,
 		const config* level, display& disp, game_state& state_of_game,
-		const std::vector<config*>& story, upload_log& log, bool skip_replay)
+		const std::vector<config*>& story, upload_log& log, bool skip_replay, end_level_exception *end_level)
 {
 	const int ticks = SDL_GetTicks();
 	const int num_turns = atoi((*level)["turns"].c_str());
 	LOG_NG << "creating objects... " << (SDL_GetTicks() - ticks) << "\n";
-	playsingle_controller playcontroller(*level, gameinfo, state_of_game, ticks, num_turns, game_config, disp.video(), skip_replay);
+	playsingle_controller playcontroller(*level, state_of_game, ticks, num_turns, game_config, disp.video(), skip_replay);
 	LOG_NG << "created objects... " << (SDL_GetTicks() - playcontroller.get_ticks()) << "\n";
 
-	const LEVEL_RESULT res = playcontroller.play_scenario(story, log, skip_replay);
+	const LEVEL_RESULT res = playcontroller.play_scenario(story, log, skip_replay, end_level);
 
 	if (res == DEFEAT) {
 		gui::message_dialog(disp,
@@ -140,7 +136,7 @@ LEVEL_RESULT playsingle_scenario(const game_data& gameinfo, const config& game_c
 				    ).show();
 	}
 
-	if (!disp.video().faked() && res != QUIT && res != LEVEL_CONTINUE && res != LEVEL_CONTINUE_NO_SAVE)
+	if (!disp.video().faked() && res != QUIT && end_level->linger_mode)
 		try {
 			playcontroller.linger(log);
 		} catch(end_level_exception& e) {
@@ -153,16 +149,16 @@ LEVEL_RESULT playsingle_scenario(const game_data& gameinfo, const config& game_c
 }
 
 
-LEVEL_RESULT playmp_scenario(const game_data& gameinfo, const config& game_config,
+static LEVEL_RESULT playmp_scenario(const config& game_config,
 		config const* level, display& disp, game_state& state_of_game,
 		const config::child_list& story, upload_log& log, bool skip_replay,
-		io_type_t& io_type)
+		io_type_t& io_type, end_level_exception *end_level)
 {
 	const int ticks = SDL_GetTicks();
 	const int num_turns = atoi((*level)["turns"].c_str());
-	playmp_controller playcontroller(*level, gameinfo, state_of_game, ticks, num_turns, 
+	playmp_controller playcontroller(*level, state_of_game, ticks, num_turns,
 		game_config, disp.video(), skip_replay, io_type == IO_SERVER);
-	const LEVEL_RESULT res = playcontroller.play_scenario(story, log, skip_replay);
+	const LEVEL_RESULT res = playcontroller.play_scenario(story, log, skip_replay, end_level);
 
 	//Check if the player started as mp client and changed to host
 	if (io_type == IO_CLIENT && playcontroller.is_host())
@@ -176,10 +172,10 @@ LEVEL_RESULT playmp_scenario(const game_data& gameinfo, const config& game_confi
 	}
 
 	if (!disp.video().faked() && res != QUIT) {
-		if(res == LEVEL_CONTINUE || res == LEVEL_CONTINUE_NO_SAVE) {
+		if(!end_level->linger_mode) {
 			if(!playcontroller.is_host()) {
-				// If we continue without lingering we need to 
-				// make sure the host uploads the next scenario 
+				// If we continue without lingering we need to
+				// make sure the host uploads the next scenario
 				// before we attempt to download it.
 				playcontroller.wait_for_upload();
 			}
@@ -198,8 +194,7 @@ LEVEL_RESULT playmp_scenario(const game_data& gameinfo, const config& game_confi
 }
 
 LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_config,
-		const game_data& units_data, upload_log &log,
-		io_type_t io_type, bool skip_replay)
+		upload_log &log, io_type_t io_type, bool skip_replay)
 {
 	std::string type = gamestate.campaign_type;
 	if(type.empty())
@@ -219,8 +214,8 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 		gamestate.completion = "running";
 		recorder.set_save_info_completion(gamestate.completion);
 		// Campaign or Multiplayer?
-		// If the gamestate already contains a starting_pos, 
-		// then we are starting a fresh multiplayer game. 
+		// If the gamestate already contains a starting_pos,
+		// then we are starting a fresh multiplayer game.
 		// Otherwise this is the start of a campaign scenario.
 		if(gamestate.starting_pos.empty() == false) {
 			LOG_G << "loading starting position...\n";
@@ -251,7 +246,7 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 			gamestate.label = gamestate.snapshot["label"];
 		}
 		{
-			// Get the current gold values of players, so they don't start 
+			// Get the current gold values of players, so they don't start
 			// with the amount they had at the start of the scenario
 			const std::vector<config*>& player_cfg = gamestate.snapshot.get_children("player");
 			for (std::vector<config*>::const_iterator p = player_cfg.begin(); p != player_cfg.end(); p++){
@@ -294,7 +289,7 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 			if(id.empty())
 				continue;
 			controllers[id] = player_controller((**side)["controller"],
-					(**side)["description"]);
+					(**side)["id"]);
 		}
 	}
 
@@ -311,10 +306,10 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 					side != sides_list.end(); ++side) {
 				if((**side)["current_player"] == preferences::login()) {
 					(**side)["controller"] = preferences::client_type();
-					(**side)["persistent"] = "1";
 				} else if((**side)["controller"] != "null") {
+					// @todo: Fix logic to use network_ai controller
+					// if it is networked ai
 					(**side)["controller"] = "network";
-					(**side)["persistent"] = "0";
 				}
 			}
 		}
@@ -324,8 +319,8 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 
 		bool save_game_after_scenario = true;
 
-		const set_random_generator generator_setter(&recorder);
-		LEVEL_RESULT res = LEVEL_CONTINUE;
+		LEVEL_RESULT res = VICTORY;
+		end_level_exception *end_level = new end_level_exception(VICTORY);
 
 		try {
 			// Preserve old label eg. replay
@@ -361,8 +356,8 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 				const cursor::setter cursor_setter(cursor::WAIT);
 				map_data = random_generate_map((*scenario)["map_generation"],scenario->child("generator"));
 
-				// Since we've had to generate the map, 
-				// make sure that when we save the game, 
+				// Since we've had to generate the map,
+				// make sure that when we save the game,
 				// it will not ask for the map to be generated again on reload
 				static config new_level;
 				new_level = *scenario;
@@ -373,15 +368,15 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 				LOG_G << "generated map\n";
 			}
 
-			sound::play_no_music();
+			sound::empty_playlist();
 
 			switch (io_type){
 			case IO_NONE:
-				res = playsingle_scenario(units_data,game_config,scenario,disp,gamestate,story,log, skip_replay);
+				res = playsingle_scenario(game_config,scenario,disp,gamestate,story,log, skip_replay, end_level);
 				break;
 			case IO_SERVER:
 			case IO_CLIENT:
-				res = playmp_scenario(units_data,game_config,scenario,disp,gamestate,story,log, skip_replay, io_type);
+				res = playmp_scenario(game_config,scenario,disp,gamestate,story,log, skip_replay, io_type, end_level);
 				break;
 			}
 		} catch(game::load_game_failed& e) {
@@ -390,7 +385,7 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 		} catch(game::game_error& e) {
 			gui::show_error_message(disp, _("Error while playing the game: ") + e.message);
 			return QUIT;
-		} catch(gamemap::incorrect_format_exception& e) {
+		} catch(incorrect_map_format_exception& e) {
 			gui::show_error_message(disp, std::string(_("The game map could not be loaded: ")) + e.msg_);
 			return QUIT;
 		} catch(config::error& e) {
@@ -407,14 +402,14 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 		// Save-nagement ioptions fire on game end.
 		// This means: (a) we have a victory, or
 		// or (b) we're multiplayer live, in which
-		// case defeat is also game end.  Someday, 
+		// case defeat is also game end.  Someday,
 		// if MP campaigns ever work again, we might
 		// need to change this test.
 
 		// OBSERVER_END probably can be removed here if the observer disconnect
 		// bug (#10077) is fixed so that when the host ends the game observers
 		// again get asked if they want to save a replay of the game
-		if (res == VICTORY || io_type != IO_NONE && (res == DEFEAT || res == OBSERVER_END)) {
+		if (res == VICTORY || (io_type != IO_NONE && (res == DEFEAT || res == OBSERVER_END))) {
 			if (preferences::delete_saves())
 				clean_saves(gamestate.label);
 
@@ -424,6 +419,13 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 					gui::OK_CANCEL, _("Save Replay"), false, false) == 0) {
 				try {
 						config snapshot;
+						//If the starting position contains player information, use this for
+						//the replay savegame (this originally comes from the gamestate constructor,
+						//where the player stuff is added to the starting position to be used here.
+						config::child_list player_list = gamestate.starting_pos.get_children("player");
+						if (player_list.size() > 0) {
+							recorder.set_save_info(gamestate, player_list);
+						}
 						recorder.save_game(label, snapshot, gamestate.starting_pos);
 					} catch(game::save_game_failed&) {
 						gui::show_error_message(disp, _("The replay could not be saved"));
@@ -436,8 +438,7 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 		gamestate.replay_data.clear();
 
 		// On DEFEAT, QUIT, or OBSERVER_END, we're done now
-		if (res != VICTORY && res != LEVEL_CONTINUE_NO_SAVE 
-			&& res != LEVEL_CONTINUE)
+		if (res != VICTORY)
 		{
 			if (res != OBSERVER_END || gamestate.next_scenario.empty())
 				return res;
@@ -449,14 +450,14 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 				return res;
 		}
 
-		// Continue without saving is like a victory, 
+		// Continue without saving is like a victory,
 		// but the save game dialog isn't displayed
-		if(res == LEVEL_CONTINUE_NO_SAVE)
+		if(!end_level->save)
 			save_game_after_scenario = false;
 
 		// Switch to the next scenario.
 		gamestate.scenario = gamestate.next_scenario;
-		gamestate.rotate_random();
+		gamestate.rng().rotate_random();
 
 		if(io_type == IO_CLIENT) {
 			if (gamestate.next_scenario.empty()) return res;
@@ -475,7 +476,7 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 			if(cfg.child("next_scenario")) {
 				starting_pos = (*cfg.child("next_scenario"));
 				scenario = &starting_pos;
-				gamestate = game_state(units_data, starting_pos);
+				gamestate = game_state(starting_pos);
 			} else {
 				return QUIT;
 			}
@@ -495,26 +496,60 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 						continue;
 					}
 
-					/* Update side info to match current_player info 
-					 * to allow it taking the side in next scenario 
-					 * and to be set in the players list on side server 
+					/* Update side info to match current_player info
+					 * to allow it taking the side in next scenario
+					 * and to be set in the players list on side server
 					 */
 					controller_map::const_iterator ctr = controllers.find(id);
 					if(ctr != controllers.end()) {
 						player_info *player = gamestate.get_player(id);
 						if (player) {
 							(**side)["current_player"] = player->name;
-							//! @todo TODO : remove (see TODO line 276 in server/game.cpp)
-							(**side)["user_description"] = player->name;
 						}
 						(**side)["controller"] = ctr->second.controller;
 					}
 				}
 
+				// If the entire scenario should be randomly generated
+				if((*scenario)["scenario_generation"] != "") {
+					LOG_G << "randomly generating scenario...\n";
+					const cursor::setter cursor_setter(cursor::WAIT);
+
+					static config scenario2;
+					scenario2 = random_generate_scenario((*scenario)["scenario_generation"], scenario->child("generator"));
+					//level_ = scenario;
+
+					gamestate.starting_pos = scenario2;
+					scenario = &scenario2;
+				}
+				std::string map_data = (*scenario)["map_data"];
+				if(map_data.empty() && (*scenario)["map"] != "") {
+					map_data = read_map((*scenario)["map"]);
+				}
+
+				// If the map should be randomly generated
+				if(map_data.empty() && (*scenario)["map_generation"] != "") {
+					const cursor::setter cursor_setter(cursor::WAIT);
+					map_data = random_generate_map((*scenario)["map_generation"],scenario->child("generator"));
+
+					// Since we've had to generate the map,
+					// make sure that when we save the game,
+					// it will not ask for the map to be generated again on reload
+					static config new_level;
+					new_level = *scenario;
+					new_level.values["map_data"] = map_data;
+					scenario = &new_level;
+
+					gamestate.starting_pos = new_level;
+					LOG_G << "generated map\n";
+				}
+
 				// Sends scenario data
 				config cfg;
 				cfg.add_child("store_next_scenario", *scenario);
-
+				//Add the player section to the starting position so we can get the correct recall list
+				//when loading the replay later on
+				write_players(gamestate, gamestate.starting_pos);
 				// Adds player information, and other state
 				// information, to the configuration object
 				assert(cfg.child("store_next_scenario") != NULL);
@@ -538,7 +573,7 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 			if(save_game_after_scenario) {
 
 				// For multiplayer, we want the save
-				// to contain the starting position.  
+				// to contain the starting position.
 				// For campaigns however, this is the
 				// start-of-scenario save and the
 				// starting position needs to be empty,
