@@ -1,7 +1,7 @@
-/* $Id: playcampaign.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: playcampaign.hpp 31859 2009-01-01 10:28:26Z mordante $ */
 /*
    Copyright (C) 2003-2005 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2008 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 - 2009 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -13,18 +13,17 @@
    See the COPYING file for more details.
 */
 
-//! @file playcampaign.hpp 
-//!
+/** @file playcampaign.hpp */
 
 #ifndef PLAYCAMPAIGN_H_INCLUDED
 #define PLAYCAMPAIGN_H_INCLUDED
 
-#include "menu_events.hpp"
+#include "game_end_exceptions.hpp"
 
 class display;
 class game_state;
 class config;
-class game_data;
+class CVideo;
 struct upload_log;
 
 enum io_type_t {
@@ -33,17 +32,15 @@ enum io_type_t {
 	IO_CLIENT
 };
 
-LEVEL_RESULT play_game(display& disp, game_state& state, 
+LEVEL_RESULT play_game(display& disp, game_state& state,
 		const config& game_config,
-		const game_data& units_data, 
 		upload_log &log,
 		io_type_t io_type=IO_NONE,
 		bool skip_replay = false);
 
 
-void play_replay(display& disp, game_state& state, 
-		const config& game_config,
-		const game_data& units_data, CVideo& video);
+void play_replay(display& disp, game_state& state,
+		const config& game_config, CVideo& video);
 
 #endif // PLAYCAMPAIGN_H_INCLUDED
 
