@@ -1,6 +1,6 @@
-/* $Id: playmp_controller.hpp 27532 2008-06-27 16:44:25Z suokko $ */
+/* $Id: playmp_controller.hpp 32690 2009-02-08 22:15:06Z jhinrichs $ */
 /*
-   Copyright (C) 2006 - 2008 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -28,12 +28,12 @@ class turn_info;
 class playmp_controller : public playsingle_controller, public events::pump_monitor
 {
 public:
-	playmp_controller(const config& level, const game_data& gameinfo, game_state& state_of_game,
-		const int ticks, const int num_turns, const config& game_config, CVideo& video, 
+	playmp_controller(const config& level, game_state& state_of_game,
+		const int ticks, const int num_turns, const config& game_config, CVideo& video,
 		bool skip_replay, bool is_host);
-	~playmp_controller();
+	virtual ~playmp_controller();
 
-	const bool is_host() { return is_host_; }
+	bool is_host() { return is_host_; }
 
 	static unsigned int replay_last_turn() { return replay_last_turn_; }
 	static void set_replay_last_turn(unsigned int turn);
@@ -43,7 +43,7 @@ public:
 	void think_about_countdown(int ticks);
 	void process(events::pump_info &info);
 	void linger(upload_log& log);
-	//! Wait for the host to upload the next scenario.
+	/** Wait for the host to upload the next scenario. */
 	void wait_for_upload();
 
 protected:
@@ -53,6 +53,8 @@ protected:
 	virtual void whisper();
 	virtual void shout();
 	virtual void clear_labels();
+	virtual void start_network();
+	virtual void stop_network();
 	virtual bool can_execute_command(hotkey::HOTKEY_COMMAND command, int index=-1) const;
 
 	virtual void play_side(const unsigned int team_index, bool save);
@@ -65,6 +67,7 @@ protected:
 	turn_info* turn_data_;
 
 	int beep_warning_time_;
+	mutable bool network_processing_stopped_;
 private:
 	void process_oos(const std::string& err_msg);
 	void set_end_scenario_button();
