@@ -1,6 +1,6 @@
-/* $Id: playturn.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: playturn.hpp 31858 2009-01-01 10:27:41Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -11,6 +11,7 @@
 
    See the COPYING file for more details.
 */
+
 #ifndef PLAYTURN_HPP_INCLUDED
 #define PLAYTURN_HPP_INCLUDED
 
@@ -33,7 +34,7 @@ class unit;
 class turn_info
 {
 public:
-	turn_info(const game_data& gameinfo, game_state& state_of_game,
+	turn_info(game_state& state_of_game,
 	          const gamestatus& status, game_display& gui, gamemap& map,
 		  std::vector<team>& teams, unsigned int team_num, unit_map& units,
 		  replay_network_sender& network_sender, undo_list& undo_stack);
@@ -44,12 +45,12 @@ public:
 
 	void send_data();
 
-	enum PROCESS_DATA_RESULT { 
-		PROCESS_CONTINUE, 
-		PROCESS_RESTART_TURN, 
-		PROCESS_END_TURN, 
-		//! When the host uploaded the next scenario this is returned.
-		PROCESS_END_LINGER        
+	enum PROCESS_DATA_RESULT {
+		PROCESS_CONTINUE,
+		PROCESS_RESTART_TURN,
+		PROCESS_END_TURN,
+		/** When the host uploaded the next scenario this is returned. */
+		PROCESS_END_LINGER
 		};
 
 	//function which will process incoming network data, and act on it. If there is
@@ -64,9 +65,9 @@ public:
 	events::generic_event& replay_error() { return replay_error_; }
 	events::generic_event& host_transfer() { return host_transfer_; }
 private:
+	static void change_controller(const std::string& side, const std::string& controller);
 	static void change_side_controller(const std::string& side, const std::string& player, bool own_side=false);
 
-	const game_data& gameinfo_;
 	game_state& state_of_game_;
 	const gamestatus& status_;
 	game_display& gui_;
