@@ -1,6 +1,6 @@
-/* $Id: preferences.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: preferences.cpp 33440 2009-03-08 16:48:17Z baufo $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,30 +12,24 @@
    See the COPYING file for more details.
 */
 
-//! @file preferences.cpp 
-//! Get and set user-preferences.
+/**
+ *  @file preferences.cpp
+ *  Get and set user-preferences.
+ */
 
 #include "global.hpp"
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
-#include "config.hpp"
 #include "filesystem.hpp"
-#include "gettext.hpp"
+#include "gui/widgets/settings.hpp"
 #include "hotkeys.hpp"
-#include "log.hpp"
 #include "preferences.hpp"
 #include "sound.hpp"
-#include "util.hpp"
 #include "video.hpp" // non_interactive()
-#include "wesconfig.h"
 #include "serialization/parser.hpp"
-#include "serialization/string_utils.hpp"
 
-#include <cstdlib>
-#include <iostream>
-#include <iterator>
-#include <sstream>
+#include <sys/stat.h> // for setting the permissions of the preferences file
 
 namespace {
 
@@ -61,11 +55,22 @@ base_manager::base_manager()
 base_manager::~base_manager()
 {
 	if (no_preferences_save) return;
+
+	// Set the 'hidden' preferences.
+	prefs["scroll_threshold"] =
+			lexical_cast<std::string>(mouse_scroll_threshold());
+
 	write_preferences();
 }
 
 void write_preferences()
 {
+    #ifndef _WIN32
+
+    bool prefs_file_existed = access(get_prefs_file().c_str(), F_OK) == 0;
+
+    #endif
+
 	try {
 		scoped_ostream prefs_file = ostream_file(get_prefs_file());
 		write(*prefs_file, prefs);
@@ -73,17 +78,46 @@ void write_preferences()
 		std::cerr << "error writing to preferences file '" << get_prefs_file() << "'\n";
 	}
 
+
+    #ifndef _WIN32
+
+    if(!prefs_file_existed) {
+
+        if(chmod(get_prefs_file().c_str(), 0600) == -1) {
+            std::cerr << "error setting permissions of preferences file '" << get_prefs_file() << "'\n";
+        }
+
+    }
+
+    #endif
+
+
 }
 
-void set(const std::string key, std::string value) {
+void set(const std::string& key, std::string value) {
 	prefs[key] = value;
 }
 
-void erase(const std::string key) {
+void clear(const std::string& key)
+{
+	prefs.recursive_clear_value(key);
+}
+
+void set_child(const std::string& key, const config& val) {
+	prefs.clear_children(key);
+	prefs.add_child(key, val);
+}
+
+config* get_child(const std::string& key)
+{
+	return prefs.child(key);
+}
+
+void erase(const std::string& key) {
 	prefs.values.erase(key);
 }
 
-const std::string get(const std::string key) {
+std::string get(const std::string& key) {
 	return prefs[key];
 }
 
@@ -103,7 +137,50 @@ bool fullscreen()
 
 void _set_fullscreen(bool ison)
 {
-	prefs["fullscreen"] = (ison ? "true" : "false");
+	prefs["fullscreen"] = (ison ? "yes" : "no");
+}
+
+bool scroll_to_action()
+{
+	return utils::string_bool(get("scroll_to_action"), true);
+}
+
+void _set_scroll_to_action(bool ison)
+{
+	prefs["scroll_to_action"] = (ison ? "yes" : "no");
+}
+
+int min_allowed_width()
+{
+#ifdef USE_TINY_GUI
+	return 320;
+#endif
+
+	// Most things (not all) in the new widgets library work properly on tiny
+	// gui resolution. So allow them, which makes initial testing easier.
+	if(gui2::new_widgets) {
+		return 320;
+	}
+
+	return 800;
+}
+
+int min_allowed_height()
+{
+#ifdef USE_TINY_GUI
+	return 240;
+#endif
+
+	// Most things (not all) in the new widgets library work properly on tiny
+	// gui resolution. So allow them, which makes initial testing easier.
+	if(gui2::new_widgets) {
+		return 240;
+	}
+
+	if (game_config::small_gui)
+		return 480;
+
+	return 600;
 }
 
 std::pair<int,int> resolution()
@@ -113,8 +190,8 @@ std::pair<int,int> resolution()
 	const string_map::const_iterator y = prefs.values.find('y' + postfix);
 	if(x != prefs.values.end() && y != prefs.values.end() &&
 	   x->second.empty() == false && y->second.empty() == false) {
-		std::pair<int,int> res (maximum(atoi(x->second.c_str()),min_allowed_width),
-		                        maximum(atoi(y->second.c_str()),min_allowed_height));
+		std::pair<int,int> res (std::max(atoi(x->second.c_str()),min_allowed_width()),
+		                        std::max(atoi(y->second.c_str()),min_allowed_height()));
 
 		// Make sure resolutions are always divisible by 4
 		//res.first &= ~3;
@@ -136,7 +213,7 @@ bool turbo()
 
 void _set_turbo(bool ison)
 {
-	prefs["turbo"] = (ison ? "true" : "false");
+	prefs["turbo"] = (ison ? "yes" : "no");
 }
 
 double turbo_speed()
@@ -179,32 +256,6 @@ void set_language(const std::string& s)
 	preferences::set("locale", s);
 }
 
-bool adjust_gamma()
-{
-	return utils::string_bool(get("adjust_gamma"), false);
-}
-
-void _set_adjust_gamma(bool val)
-{
-	preferences::set("adjust_gamma", val ? "yes" : "no");
-}
-
-int gamma()
-{
-	static const int default_value = 100;
-
-	if(!adjust_gamma()) {
-		return default_value;
-	}
-
-	return lexical_cast_default<int>(get("gamma"), default_value);
-}
-
-void _set_gamma(int gamma)
-{
-	preferences::set("gamma", lexical_cast<std::string>(gamma));
-}
-
 bool grid()
 {
 	return utils::string_bool(get("grid"), false);
@@ -212,14 +263,14 @@ bool grid()
 
 void _set_grid(bool ison)
 {
-	preferences::set("grid",  (ison ? "true" : "false"));
+	preferences::set("grid",  (ison ? "yes" : "no"));
 }
 
 size_t sound_buffer_size()
 {
 	// Sounds don't sound good on Windows unless the buffer size is 4k,
 	// but this seems to cause crashes on other systems...
-	#ifdef WIN32
+	#ifdef _WIN32
 		const size_t buf_size = 4096;
 	#else
 		const size_t buf_size = 1024;
@@ -230,7 +281,7 @@ size_t sound_buffer_size()
 
 void save_sound_buffer_size(const size_t size)
 {
-	#ifdef WIN32
+	#ifdef _WIN32
 		const char* buf_size = "4096";
 	#else
 		const char* buf_size = "1024";
@@ -409,6 +460,49 @@ bool set_music(bool ison) {
 
 namespace {
 	double scroll = 0.2;
+
+	void normalize_editor_rgb(int rval)
+	{
+		if (rval < -255) {
+			rval = -255;
+		}
+		else if (rval > 255) {
+			rval = 255;
+		}
+	}
+}
+
+void set_editor_r(int value)
+{
+	normalize_editor_rgb(value);
+	prefs["editor_r"] = lexical_cast<std::string>(value);
+}
+
+void set_editor_g(int value)
+{
+	normalize_editor_rgb(value);
+	prefs["editor_g"] = lexical_cast<std::string>(value);
+}
+
+void set_editor_b(int value)
+{
+	normalize_editor_rgb(value);
+	prefs["editor_b"] = lexical_cast<std::string>(value);
+}
+
+int editor_r()
+{
+	return lexical_cast_in_range<int>(get("editor_r"), 0, -255, 255);
+}
+
+int editor_g()
+{
+	return lexical_cast_in_range<int>(get("editor_g"), 0, -255, 255);
+}
+
+int editor_b()
+{
+	return lexical_cast_in_range<int>(get("editor_b"), 0, -255, 255);
 }
 
 int scroll_speed()
@@ -425,6 +519,11 @@ void set_scroll_speed(const int new_speed)
 	scroll = new_speed / 100.0;
 }
 
+bool middle_click_scrolls()
+{
+	return utils::string_bool(get("middle_click_scrolls"), true);
+}
+
 bool mouse_scroll_enabled()
 {
 	return utils::string_bool(get("mouse_scrolling"), true);
@@ -435,6 +534,11 @@ void enable_mouse_scroll(bool value)
 	set("mouse_scrolling", value ? "yes" : "no");
 }
 
+int mouse_scroll_threshold()
+{
+	return lexical_cast_default<int>(prefs["scroll_threshold"], 10);
+}
+
 bool animate_map()
 {
 	return utils::string_bool(preferences::get("animate_map"), true);
@@ -483,6 +587,19 @@ void save_hotkeys() {
 	hotkey::save_hotkeys(prefs);
 }
 
+void add_alias(const std::string& alias, const std::string& command) {
+	config* alias_list = prefs.child("alias");
+	if (alias_list == NULL) {
+		alias_list = &(prefs.add_child("alias"));
+	}
+	alias_list->values[alias] = command;
+}
+
+
+config* get_alias() {
+	return prefs.child("alias");
+}
+
 unsigned int sample_rate()
 {
 	return lexical_cast_default<unsigned int>(preferences::get("sample_rate"), 44100);
