@@ -1,7 +1,7 @@
-/* $Id: binary_or_text.cpp 26880 2008-05-26 21:36:58Z suokko $ */
+/* $Id: binary_or_text.cpp 33005 2009-02-22 12:19:57Z silene $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2005 - 2009 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,18 +13,19 @@
    See the COPYING file for more details.
 */
 
-//! @file serialization/binary_or_text.cpp 
-//! Read/Write file in binary (compressed) or text-format (uncompressed).
+/**
+ * @file serialization/binary_or_text.cpp.
+ * Read/Write file in binary (compressed) or text-format (uncompressed).
+ */
 
 #include "global.hpp"
 
 #include "binary_or_text.hpp"
 #include "config.hpp"
-#include "filesystem.hpp"
+#include "wesconfig.h"
 #include "serialization/binary_wml.hpp"
 #include "serialization/parser.hpp"
 
-#include <sstream>
 
 #include <boost/iostreams/filter/gzip.hpp>
 
@@ -49,13 +50,13 @@ void write_possibly_compressed(std::ostream &out, config &cfg, bool compress)
 }
 
 config_writer::config_writer(
-	std::ostream &out, bool compress, const std::string &textdomain,int level) :
+	std::ostream &out, bool compress, int level) :
 		filter_(),
 		out_ptr_(compress ? &filter_ : &out), //ternary indirection creates a temporary
 		out_(*out_ptr_), //now MSVC will allow binding to the reference member
-		compress_(compress), 
-		level_(0), 
-		textdomain_(textdomain)
+		compress_(compress),
+		level_(0),
+		textdomain_(PACKAGE)
 {
 	if(compress_) {
 		if (level >=0)
@@ -67,6 +68,7 @@ config_writer::config_writer(
 	}
 }
 
+
 void config_writer::write(const config &cfg)
 {
 	::write(out_, cfg, level_);
