@@ -1,4 +1,4 @@
-/* $Id: preprocessor.cpp 33310 2009-03-03 17:44:11Z silene $ */
+/* $Id: preprocessor.cpp 35988 2009-06-01 11:41:05Z silene $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
    Copyright (C) 2005 - 2009 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
@@ -166,7 +166,6 @@ class preprocessor_streambuf: public streambuf
 	preprocessor_streambuf(preprocessor_streambuf const &);
 public:
 	preprocessor_streambuf(preproc_map *, std::string *);
-	std::string lineno_string(std::string const &);
 	void error(const std::string &, const std::string &);
 };
 
@@ -251,7 +250,7 @@ int preprocessor_streambuf::underflow()
 	return static_cast<unsigned char>(*(begin + sz));
 }
 
-std::string preprocessor_streambuf::lineno_string(std::string const &lineno)
+std::string lineno_string(const std::string &lineno)
 {
 	std::vector< std::string > pos = utils::quoted_split(lineno, ' ');
 	std::vector< std::string >::const_iterator i = pos.begin(), end = pos.end();
@@ -259,10 +258,13 @@ std::string preprocessor_streambuf::lineno_string(std::string const &lineno)
 	std::string res;
 	while (i != end) {
 		std::string const &line = *(i++);
-		std::string const &file = i != end ? *(i++) : "<unknown>";
 		if (!res.empty())
 			res += included_from;
-		res += file + ':' + line;
+		if (i != end)
+			res += *(i++);
+		else
+			res += "<unknown>";
+		res += ':' + line;
 	}
 	if (res.empty()) res = "???";
 	return res;
@@ -964,7 +966,13 @@ bool preprocessor_data::get_chunk()
 					}
 				}
 				else
+				{
+					if (!slowpath_)
+						put("dummy=" + symbol);
+					else
+						strings_.back() += "dummy=" + symbol;
 					ERR_CF << "File not found '" << symbol << "'\n";
+				}
 			} else {
 				ERR_CF << "Too much nested preprocessing inclusions at "
 				       << linenum_ << ' ' << target_.location_
