@@ -1,4 +1,4 @@
-/* $Id: metrics.cpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: metrics.cpp 37581 2009-08-09 19:09:44Z soliton $ */
 /*
    Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -108,7 +108,7 @@ void metrics::game_terminated(const std::string& reason)
 	terminations_[reason]++;
 }
 
-std::ostream& metrics::games(std::ostream& out)
+std::ostream& metrics::games(std::ostream& out) const
 {
 	if (terminations_.empty()) return out << "No game ended so far.";
 
@@ -123,25 +123,11 @@ std::ostream& metrics::games(std::ostream& out)
 	return out;
 }
 
-std::ostream& operator<<(std::ostream& out, metrics& met)
+std::ostream& metrics::requests(std::ostream& out) const
 {
-	const time_t time_up = time(NULL) - met.started_at_;
-	const int seconds = time_up%60;
-	const int minutes = (time_up/60)%60;
-	const int hours = (time_up/(60*60))%24;
-	const int days = time_up/(60*60*24);
-	const int requests_immediate = met.nrequests_ - met.nrequests_waited_;
-	const int percent_immediate = (requests_immediate*100)/(met.nrequests_ > 0 ? met.nrequests_ : 1);
-	out << "METRICS\nUp " << days << " days, " << hours << " hours, "
-	    << minutes << " minutes, " << seconds << " seconds\n"
-	    << met.nrequests_ << " requests serviced. " << requests_immediate
-	    << " (" << percent_immediate << "%) "
-	    << "requests were serviced immediately.\n"
-	    << "longest burst of requests was: " << met.most_consecutive_requests_;
+	if (samples_.empty()) return out;
 
-	if (met.samples_.empty()) return out;
-
-	std::vector<metrics::sample> ordered_samples = met.samples_;
+	std::vector<metrics::sample> ordered_samples = samples_;
 	std::sort(ordered_samples.begin(), ordered_samples.end(), compare_samples_by_time());
 
 	out << "\nSampled request types:\n";
@@ -163,3 +149,22 @@ std::ostream& operator<<(std::ostream& out, metrics& met)
 
 	return out;
 }
+
+std::ostream& operator<<(std::ostream& out, metrics& met)
+{
+	const time_t time_up = time(NULL) - met.started_at_;
+	const int seconds = time_up%60;
+	const int minutes = (time_up/60)%60;
+	const int hours = (time_up/(60*60))%24;
+	const int days = time_up/(60*60*24);
+	const int requests_immediate = met.nrequests_ - met.nrequests_waited_;
+	const int percent_immediate = (requests_immediate*100)/(met.nrequests_ > 0 ? met.nrequests_ : 1);
+	out << "METRICS\nUp " << days << " days, " << hours << " hours, "
+	    << minutes << " minutes, " << seconds << " seconds\n"
+	    << met.nrequests_ << " requests serviced. " << requests_immediate
+	    << " (" << percent_immediate << "%) "
+	    << "requests were serviced immediately.\n"
+	    << "longest burst of requests was: " << met.most_consecutive_requests_;
+
+	return out;
+}
