@@ -6,6 +6,10 @@
 
 #include "simple_wml.hpp"
 
+#include "../log.hpp"
+
+#define ERR_SWML LOG_STREAM(err, config)
+
 namespace simple_wml {
 
 namespace {
@@ -23,10 +27,10 @@ char* uncompress_buffer(const string_span& input, string_span* span)
 	filter.push(boost::iostreams::gzip_decompressor());
 	filter.push(stream);
 
-	const int chunk_size = input.size() * 10;
+	const size_t chunk_size = input.size() * 10;
 	std::vector<char> buf(chunk_size);
-	int len = 0;
-	int pos = 0;
+	size_t len = 0;
+	size_t pos = 0;
 	while(filter.good() && (len = filter.read(&buf[pos], chunk_size).gcount()) == chunk_size) {
 		if(pos + chunk_size > 40000000) {
 			throw error("WML document exceeds 40MB limit");
@@ -54,26 +58,33 @@ char* uncompress_buffer(const string_span& input, string_span* span)
 
 char* compress_buffer(const char* input, string_span* span)
 {
-	std::string in(input);
-	std::istringstream stream(in);
-	boost::iostreams::filtering_stream<boost::iostreams::input> filter;
-	filter.push(boost::iostreams::gzip_compressor());
-	filter.push(stream);
+	try {
+		std::string in(input);
+		std::istringstream stream(in);
+		boost::iostreams::filtering_stream<boost::iostreams::input> filter;
+		filter.push(boost::iostreams::gzip_compressor());
+		filter.push(stream);
 
-	std::vector<char> buf(in.size()*2 + 80);
-	const int len = filter.read(&buf[0], buf.size()).gcount();
-	if((!filter.eof() && !filter.good()) || len == static_cast<int>(buf.size())) {
-		throw error("failed to compress");
+		std::vector<char> buf(in.size()*2 + 80);
+		const int len = filter.read(&buf[0], buf.size()).gcount();
+		assert(len < 128*1024*1024);
+		if((!filter.eof() && !filter.good()) || len == static_cast<int>(buf.size())) {
+			throw error("failed to compress");
+		}
+
+		buf.resize(len);
+
+		char* small_out = new char[len];
+		memcpy(small_out, &buf[0], len);
+
+		*span = string_span(small_out, len);
+		assert(*small_out == 31);
+		return small_out;
+	} catch (std::bad_alloc& e) {
+		ERR_SWML << "ERROR: bad_alloc caught in compress_buffer() with input: '"
+		<< input << "' " << e.what() << std::endl;
+		throw error("Bad allocation request in compress_buffer().");
 	}
-
-	buf.resize(len);
-
-	char* small_out = new char[len];
-	memcpy(small_out, &buf[0], len);
-
-	*span = string_span(small_out, len);
-	assert(*small_out == 31);
-	return small_out;
 }
 
 }  // namespace
@@ -115,7 +126,7 @@ char* string_span::duplicate() const
 error::error(const char* msg)
   : message(msg)
 {
-	std::cerr << "ERROR: '" << msg << "'\n";
+	ERR_SWML << "ERROR: '" << msg << "'\n";
 }
 
 std::ostream& operator<<(std::ostream& o, const string_span& s)
@@ -194,7 +205,7 @@ node::node(document& doc, node* parent, const char** str, int depth) :
 		default: {
 			const char* end = strchr(s, '=');
 			if(end == NULL) {
-				std::cerr << "attribute: " << s << "\n";
+				ERR_SWML << "attribute: " << s << "\n";
 				throw error("could not find '=' after attribute");
 			}
 
@@ -208,7 +219,7 @@ node::node(document& doc, node* parent, const char** str, int depth) :
 			}
 
 			if(*s != '"') {
-				std::cerr << "no quotes for attribute '" << name << "'\n";
+				ERR_SWML << "no quotes for attribute '" << name << "'\n";
 				throw error("did not find quotes around attribute");
 			}
 
@@ -220,7 +231,7 @@ node::node(document& doc, node* parent, const char** str, int depth) :
 				}
 
 				if(end == NULL) {
-					std::cerr << "ATTR: '" << name << "' (((" << s << ")))\n";
+					ERR_SWML << "ATTR: '" << name << "' (((" << s << ")))\n";
 					throw error("did not find end of attribute");
 				}
 
@@ -243,7 +254,7 @@ node::node(document& doc, node* parent, const char** str, int depth) :
 
 			string_span value(s, end - s);
 			if(attr_.empty() == false && !(attr_.back().first < name)) {
-				std::cerr << "attributes: '" << attr_.back().first << "' < '" << name << "'\n";
+				ERR_SWML << "attributes: '" << attr_.back().first << "' < '" << name << "'\n";
 				throw error("attributes not in order");
 			}
 
@@ -945,7 +956,14 @@ const char* document::output()
 	bufs.swap(buffers_);
 
 	const int buf_size = root_->output_size() + 1;
-	char* buf = new char[buf_size];
+	char* buf;
+	try {
+		buf = new char[buf_size];
+	} catch (std::bad_alloc& e) {
+		ERR_SWML << "ERROR: Trying to allocate " << buf_size << " bytes. "
+		<< e.what() << std::endl;
+		throw error("Bad allocation request in output().");
+	}
 	buffers_.push_back(buf);
 	output_ = buf;
 
