@@ -1,6 +1,6 @@
-/* $Id: statistics.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: statistics.cpp 33516 2009-03-10 08:37:11Z silene $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,18 +12,20 @@
    See the COPYING file for more details.
 */
 
-//! @file statistics.cpp
-//! Manage statistics: recruitments, recalls, kills, losses, etc.
+/**
+ *  @file statistics.cpp
+ *  Manage statistics: recruitments, recalls, kills, losses, etc.
+ */
 
 #include "global.hpp"
-#include "config.hpp"
 #include "statistics.hpp"
-#include "util.hpp"
 #include "log.hpp"
 #include "serialization/binary_or_text.hpp"
 #include "unit.hpp"
+#include "util.hpp"
 
 #define ERR_NG lg::err(lg::engine)
+#define DBG_NG LOG_STREAM(debug, engine)
 
 namespace {
 
@@ -37,7 +39,9 @@ int stats_disabled = 0;
 
 struct scenario_stats
 {
-	explicit scenario_stats(const std::string& name) : scenario_name(name)
+	explicit scenario_stats(const std::string& name) :
+		team_stats(),
+		scenario_name(name)
 	{}
 
 	explicit scenario_stats(const config& cfg);
@@ -45,16 +49,17 @@ struct scenario_stats
 	config write() const;
 	void write(config_writer &out) const;
 
-	std::vector<stats> team_stats;
+	std::map<std::string,stats> team_stats;
 	std::string scenario_name;
 };
 
-scenario_stats::scenario_stats(const config& cfg)
+scenario_stats::scenario_stats(const config& cfg) :
+	team_stats(),
+	scenario_name(cfg["scenario"])
 {
-	scenario_name = cfg["scenario"];
 	const config::child_list& teams = cfg.get_children("team");
 	for(config::child_list::const_iterator i = teams.begin(); i != teams.end(); ++i) {
-		team_stats.push_back(stats(**i));
+		team_stats[(**i)["save_id"]] = stats(**i);
 	}
 }
 
@@ -62,8 +67,8 @@ config scenario_stats::write() const
 {
 	config res;
 	res["scenario"] = scenario_name;
-	for(std::vector<stats>::const_iterator i = team_stats.begin(); i != team_stats.end(); ++i) {
-		res.add_child("team",i->write());
+	for(std::map<std::string,stats>::const_iterator i = team_stats.begin(); i != team_stats.end(); ++i) {
+		res.add_child("team",i->second.write());
 	}
 
 	return res;
@@ -72,9 +77,9 @@ config scenario_stats::write() const
 void scenario_stats::write(config_writer &out) const
 {
 	out.write_key_val("scenario", scenario_name);
-	for(std::vector<stats>::const_iterator i = team_stats.begin(); i != team_stats.end(); ++i) {
+	for(std::map<std::string,stats>::const_iterator i = team_stats.begin(); i != team_stats.end(); ++i) {
 		out.open_child("team");
-		i->write(out);
+		i->second.write(out);
 		out.close_child("team");
 	}
 }
@@ -83,19 +88,14 @@ std::vector<scenario_stats> master_stats;
 
 } // end anon namespace
 
-static stats& get_stats(int team)
+static stats &get_stats(const std::string &save_id)
 {
 	if(master_stats.empty()) {
-		master_stats.push_back(scenario_stats(""));
+		master_stats.push_back(scenario_stats(std::string()));
 	}
 
-	std::vector<stats>& team_stats = master_stats.back().team_stats;
-	const size_t index = size_t(team-1);
-	if(index >= team_stats.size()) {
-		team_stats.resize(index+1);
-	}
-
-	return team_stats[index];
+	std::map<std::string,stats>& team_stats = master_stats.back().team_stats;
+	return team_stats[save_id];
 }
 
 static config write_str_int_map(const stats::str_int_map& m)
@@ -187,6 +187,7 @@ static void merge_battle_result_maps(stats::battle_result_map& a, const stats::b
 
 static void merge_stats(stats& a, const stats& b)
 {
+	DBG_NG << "Merging statistics\n";
 	merge_str_int_map(a.recruits,b.recruits);
 	merge_str_int_map(a.recalls,b.recalls);
 	merge_str_int_map(a.advanced_to,b.advanced_to);
@@ -203,23 +204,56 @@ static void merge_stats(stats& a, const stats& b)
 	a.damage_taken += b.damage_taken;
 	a.expected_damage_inflicted += b.expected_damage_inflicted;
 	a.expected_damage_taken += b.expected_damage_taken;
-	a.turn_damage_inflicted += b.turn_damage_inflicted;
-	a.turn_damage_taken += b.turn_damage_taken;
-	a.turn_expected_damage_inflicted += b.turn_expected_damage_inflicted;
-	a.turn_expected_damage_taken += b.turn_expected_damage_taken;
+	// Only take the last value for this turn
+	a.turn_damage_inflicted = b.turn_damage_inflicted;
+	a.turn_damage_taken = b.turn_damage_taken;
+	a.turn_expected_damage_inflicted = b.turn_expected_damage_inflicted;
+	a.turn_expected_damage_taken = b.turn_expected_damage_taken;
 }
 
 namespace statistics
 {
 
-stats::stats() : recruit_cost(0), recall_cost(0),
-                 damage_inflicted(0), damage_taken(0),
-                 turn_damage_inflicted(0), turn_damage_taken(0),
-                 expected_damage_inflicted(0), expected_damage_taken(0),
-                 turn_expected_damage_inflicted(0), turn_expected_damage_taken(0)
+stats::stats() :
+	recruits(),
+	recalls(),
+	advanced_to(),
+	deaths(),
+	killed(),
+	recruit_cost(0),
+	recall_cost(0),
+	attacks(),
+	defends(),
+	damage_inflicted(0),
+	damage_taken(0),
+	turn_damage_inflicted(0),
+	turn_damage_taken(0),
+	expected_damage_inflicted(0),
+	expected_damage_taken(0),
+	turn_expected_damage_inflicted(0),
+	turn_expected_damage_taken(0),
+	save_id()
 {}
 
-stats::stats(const config& cfg)
+stats::stats(const config& cfg) :
+	recruits(),
+	recalls(),
+	advanced_to(),
+	deaths(),
+	killed(),
+	recruit_cost(0),
+	recall_cost(0),
+	attacks(),
+	defends(),
+	damage_inflicted(0),
+	damage_taken(0),
+	turn_damage_inflicted(0),
+	turn_damage_taken(0),
+	expected_damage_inflicted(0),
+	expected_damage_taken(0),
+	turn_expected_damage_inflicted(0),
+	turn_expected_damage_taken(0),
+	save_id(std::string())
 {
 	read(cfg);
 }
@@ -235,29 +269,38 @@ config stats::write() const
 	res.add_child("attacks",write_battle_result_map(attacks));
 	res.add_child("defends",write_battle_result_map(defends));
 
-	char buf[50];
-	snprintf(buf,sizeof(buf),"%d",recruit_cost);
-	res["recruit_cost"] = buf;
-	snprintf(buf,sizeof(buf),"%d",recall_cost);
-	res["recall_cost"] = buf;
+	std::ostringstream ss;
+	ss << recruit_cost;
+	res["recruit_cost"] = ss.str();
+	ss.str(std::string());
+	ss << recall_cost;
+	res["recall_cost"] = ss.str();
 
-	snprintf(buf,sizeof(buf),"%d",damage_inflicted);
-	res["damage_inflicted"] = buf;
-	snprintf(buf,sizeof(buf),"%d",damage_taken);
-	res["damage_taken"] = buf;
-	snprintf(buf,sizeof(buf),"%d",expected_damage_inflicted);
-	res["expected_damage_inflicted"] = buf;
-	snprintf(buf,sizeof(buf),"%d",expected_damage_taken);
-	res["expected_damage_taken"] = buf;
+	ss.str(std::string());
+	ss << damage_inflicted;
+	res["damage_inflicted"] = ss.str();
+	ss.str(std::string());
+	ss << damage_taken;
+	res["damage_taken"] = ss.str();
+	ss.str(std::string());
+	ss << expected_damage_inflicted;
+	res["expected_damage_inflicted"] = ss.str();
+	ss.str(std::string());
+	ss << expected_damage_taken;
+	res["expected_damage_taken"] = ss.str();
+	ss.str(std::string());
+	ss << turn_damage_inflicted;
 
-	snprintf(buf,sizeof(buf),"%d",turn_damage_inflicted);
-	res["turn_damage_inflicted"] = buf;
-	snprintf(buf,sizeof(buf),"%d",turn_damage_taken);
-	res["turn_damage_taken"] = buf;
-	snprintf(buf,sizeof(buf),"%d",turn_expected_damage_inflicted);
-	res["turn_expected_damage_inflicted"] = buf;
-	snprintf(buf,sizeof(buf),"%d",turn_expected_damage_taken);
-	res["turn_expected_damage_taken"] = buf;
+	res["turn_damage_inflicted"] = ss.str();
+	ss.str(std::string());
+	ss << turn_damage_taken;
+	res["turn_damage_taken"] = ss.str();
+	ss.str(std::string());
+	ss << turn_expected_damage_inflicted;
+	res["turn_expected_damage_inflicted"] = ss.str();
+	ss.str(std::string());
+	ss << turn_expected_damage_taken;
+	res["turn_expected_damage_taken"] = ss.str();
 
 	return res;
 }
@@ -286,29 +329,41 @@ void stats::write(config_writer &out) const
 	write_battle_result_map(out, defends);
 	out.close_child("defends");
 
-	char buf[50];
-	snprintf(buf,sizeof(buf),"%d",recruit_cost);
-	out.write_key_val("recruit_cost", buf);
-	snprintf(buf,sizeof(buf),"%d",recall_cost);
-	out.write_key_val("recall_cost", buf);
+	std::ostringstream ss;
+	ss << recruit_cost;
+	out.write_key_val("recruit_cost", ss.str());
+	ss.str(std::string());
+	ss << recall_cost;
+	out.write_key_val("recall_cost", ss.str());
 
-	snprintf(buf,sizeof(buf),"%d",damage_inflicted);
-	out.write_key_val("damage_inflicted", buf);
-	snprintf(buf,sizeof(buf),"%d",damage_taken);
-	out.write_key_val("damage_taken", buf);
-	snprintf(buf,sizeof(buf),"%d",expected_damage_inflicted);
-	out.write_key_val("expected_damage_inflicted", buf);
-	snprintf(buf,sizeof(buf),"%d",expected_damage_taken);
-	out.write_key_val("expected_damage_taken", buf);
+	ss.str(std::string());
+	ss << damage_inflicted;
+	out.write_key_val("damage_inflicted", ss.str());
+	ss.str(std::string());
+	ss << damage_taken;
+	out.write_key_val("damage_taken", ss.str());
+	ss.str(std::string());
+	ss << expected_damage_inflicted;
+	out.write_key_val("expected_damage_inflicted", ss.str());
+	ss.str(std::string());
+	ss << expected_damage_taken;
+	out.write_key_val("expected_damage_taken", ss.str());
+	ss.str(std::string());
+	ss << turn_damage_inflicted;
+
+	out.write_key_val("turn_damage_inflicted", ss.str());
+	ss.str(std::string());
+	ss << turn_damage_taken;
+	out.write_key_val("turn_damage_taken", ss.str());
+	ss.str(std::string());
+	ss << turn_expected_damage_inflicted;
+	out.write_key_val("turn_expected_damage_inflicted", ss.str());
+	ss.str(std::string());
+	ss << turn_expected_damage_taken;
+	out.write_key_val("turn_expected_damage_taken", ss.str());
+
+	out.write_key_val("save_id", save_id);
 
-	snprintf(buf,sizeof(buf),"%d",turn_damage_inflicted);
-	out.write_key_val("turn_damage_inflicted", buf);
-	snprintf(buf,sizeof(buf),"%d",turn_damage_taken);
-	out.write_key_val("turn_damage_taken", buf);
-	snprintf(buf,sizeof(buf),"%d",turn_expected_damage_inflicted);
-	out.write_key_val("turn_expected_damage_inflicted", buf);
-	snprintf(buf,sizeof(buf),"%d",turn_expected_damage_taken);
-	out.write_key_val("turn_expected_damage_taken", buf);
 }
 
 void stats::read(const config& cfg)
@@ -338,18 +393,20 @@ void stats::read(const config& cfg)
 		defends = read_battle_result_map(*cfg.child("defends"));
 	}
 
-	recruit_cost = atoi(cfg["recruit_cost"].c_str());
-	recall_cost = atoi(cfg["recall_cost"].c_str());
+	recruit_cost = lexical_cast<long long>(cfg["recruit_cost"]);
+	recall_cost = lexical_cast<long long>(cfg["recall_cost"]);
 
-	damage_inflicted = atoi(cfg["damage_inflicted"].c_str());
-	damage_taken = atoi(cfg["damage_taken"].c_str());
-	expected_damage_inflicted = atoi(cfg["expected_damage_inflicted"].c_str());
-	expected_damage_taken = atoi(cfg["expected_damage_taken"].c_str());
+	damage_inflicted = lexical_cast<long long>(cfg["damage_inflicted"]);
+	damage_taken = lexical_cast<long long>(cfg["damage_taken"]);
+	expected_damage_inflicted = lexical_cast<long long>(cfg["expected_damage_inflicted"]);
+	expected_damage_taken = lexical_cast<long long>(cfg["expected_damage_taken"]);
 
-	turn_damage_inflicted = atoi(cfg["turn_damage_inflicted"].c_str());
-	turn_damage_taken = atoi(cfg["turn_damage_taken"].c_str());
-	turn_expected_damage_inflicted = atoi(cfg["turn_expected_damage_inflicted"].c_str());
-	turn_expected_damage_taken = atoi(cfg["turn_expected_damage_taken"].c_str());
+	turn_damage_inflicted = lexical_cast<long long>(cfg["turn_damage_inflicted"]);
+	turn_damage_taken = lexical_cast<long long>(cfg["turn_damage_taken"]);
+	turn_expected_damage_inflicted = lexical_cast<long long>(cfg["turn_expected_damage_inflicted"]);
+	turn_expected_damage_taken = lexical_cast<long long>(cfg["turn_expected_damage_taken"]);
+
+	save_id = cfg["save_id"];
 }
 
 disabler::disabler() { stats_disabled++; }
@@ -369,10 +426,16 @@ scenario_context::~scenario_context()
 	mid_scenario = false;
 }
 
-attack_context::attack_context(const unit& a, const unit& d, int a_cth, int d_cth)
-   : attacker_type(a.id()), defender_type(d.id()),
-     attacker_side(a.side()), defender_side(d.side()),
-     chance_to_hit_defender(a_cth), chance_to_hit_attacker(d_cth)
+attack_context::attack_context(const unit& a,
+		const unit& d, int a_cth, int d_cth) :
+	attacker_type(a.type_id()),
+	defender_type(d.type_id()),
+	attacker_side(a.side_id()),
+	defender_side(d.side_id()),
+	chance_to_hit_defender(a_cth),
+	chance_to_hit_attacker(d_cth),
+	attacker_res(),
+	defender_res()
 {
 }
 
@@ -398,53 +461,73 @@ stats& attack_context::defender_stats()
 	return get_stats(defender_side);
 }
 
-void attack_context::attack_result(attack_context::ATTACK_RESULT res, int damage)
+void attack_context::attack_expected_damage(double attacker_inflict_, double defender_inflict_)
+{
+	int attacker_inflict = round_double(attacker_inflict_ * stats::decimal_shift);
+	int defender_inflict = round_double(defender_inflict_ * stats::decimal_shift);
+	stats &att_stats = attacker_stats(), &def_stats = defender_stats();
+	att_stats.expected_damage_inflicted += attacker_inflict;
+	att_stats.expected_damage_taken     += defender_inflict;
+	def_stats.expected_damage_inflicted += defender_inflict;
+	def_stats.expected_damage_taken     += attacker_inflict;
+	att_stats.turn_expected_damage_inflicted += attacker_inflict;
+	att_stats.turn_expected_damage_taken     += defender_inflict;
+	def_stats.turn_expected_damage_inflicted += defender_inflict;
+	def_stats.turn_expected_damage_taken     += attacker_inflict;
+}
+
+
+void attack_context::attack_result(hit_result res, int damage, int drain)
 {
 	if(stats_disabled > 0)
 		return;
 
-	push_back(attacker_res,(res == MISSES ? '0' : '1'));
+	attacker_res.push_back(res == MISSES ? '0' : '1');
+	stats &att_stats = attacker_stats(), &def_stats = defender_stats();
 
 	if(res != MISSES) {
-		attacker_stats().damage_inflicted += damage;
-		defender_stats().damage_taken += damage;
-		attacker_stats().turn_damage_inflicted += damage;
-		defender_stats().turn_damage_taken += damage;
+		// handle drain
+		att_stats.damage_taken          -= drain;
+		def_stats.damage_inflicted      -= drain;
+		att_stats.turn_damage_taken     -= drain;
+		def_stats.turn_damage_inflicted -= drain;
+
+		att_stats.damage_inflicted      += damage;
+		def_stats.damage_taken          += damage;
+		att_stats.turn_damage_inflicted += damage;
+		def_stats.turn_damage_taken     += damage;
 	}
-	const int exp_damage = damage * chance_to_hit_defender;
-	attacker_stats().expected_damage_inflicted += exp_damage;
-	defender_stats().expected_damage_taken += exp_damage;
-	attacker_stats().turn_expected_damage_inflicted += exp_damage;
-	defender_stats().turn_expected_damage_taken += exp_damage;
 
 	if(res == KILLS) {
-		attacker_stats().killed[defender_type]++;
-		defender_stats().deaths[defender_type]++;
+		++att_stats.killed[defender_type];
+		++def_stats.deaths[defender_type];
 	}
 }
 
-void attack_context::defend_result(attack_context::ATTACK_RESULT res, int damage)
+void attack_context::defend_result(hit_result res, int damage, int drain)
 {
 	if(stats_disabled > 0)
 		return;
 
-	push_back(defender_res,(res == MISSES ? '0' : '1'));
+	defender_res.push_back(res == MISSES ? '0' : '1');
+	stats &att_stats = attacker_stats(), &def_stats = defender_stats();
 
 	if(res != MISSES) {
-		attacker_stats().damage_taken += damage;
-		defender_stats().damage_inflicted += damage;
-		attacker_stats().turn_damage_taken += damage;
-		defender_stats().turn_damage_inflicted += damage;
+		//handle drain
+		def_stats.damage_taken          -= drain;
+		att_stats.damage_inflicted      -= drain;
+		def_stats.turn_damage_taken     -= drain;
+		att_stats.turn_damage_inflicted -= drain;
+
+		att_stats.damage_taken          += damage;
+		def_stats.damage_inflicted      += damage;
+		att_stats.turn_damage_taken     += damage;
+		def_stats.turn_damage_inflicted += damage;
 	}
-	const int exp_damage = damage * chance_to_hit_attacker;
-	attacker_stats().expected_damage_taken += exp_damage;
-	defender_stats().expected_damage_inflicted += exp_damage;
-	attacker_stats().turn_expected_damage_taken += exp_damage;
-	defender_stats().turn_expected_damage_inflicted += exp_damage;
 
 	if(res == KILLS) {
-		attacker_stats().deaths[attacker_type]++;
-		defender_stats().killed[attacker_type]++;
+		++att_stats.deaths[attacker_type];
+		++def_stats.killed[attacker_type];
 	}
 }
 
@@ -453,8 +536,8 @@ void recruit_unit(const unit& u)
 	if(stats_disabled > 0)
 		return;
 
-	stats& s = get_stats(u.side());
-	s.recruits[u.id()]++;
+	stats& s = get_stats(u.side_id());
+	s.recruits[u.type_id()]++;
 	s.recruit_cost += u.cost();
 }
 
@@ -463,8 +546,8 @@ void recall_unit(const unit& u)
 	if(stats_disabled > 0)
 		return;
 
-	stats& s = get_stats(u.side());
-	s.recalls[u.id()]++;
+	stats& s = get_stats(u.side_id());
+	s.recalls[u.type_id()]++;
 	s.recall_cost += u.cost();
 }
 
@@ -473,8 +556,8 @@ void un_recall_unit(const unit& u)
 	if(stats_disabled > 0)
 		return;
 
-	stats& s = get_stats(u.side());
-	s.recalls[u.id()]--;
+	stats& s = get_stats(u.side_id());
+	s.recalls[u.type_id()]--;
 	s.recall_cost -= u.cost();
 }
 
@@ -483,8 +566,8 @@ void un_recruit_unit(const unit& u)
 	if(stats_disabled > 0)
 		return;
 
-	stats& s = get_stats(u.side());
-	s.recruits[u.id()]--;
+	stats& s = get_stats(u.side_id());
+	s.recruits[u.type_id()]--;
 	s.recruit_cost -= u.cost();
 }
 
@@ -494,33 +577,35 @@ void advance_unit(const unit& u)
 	if(stats_disabled > 0)
 		return;
 
-	stats& s = get_stats(u.side());
-	s.advanced_to[u.id()]++;
+	stats& s = get_stats(u.side_id());
+	s.advanced_to[u.type_id()]++;
 }
 
-void reset_turn_stats(int side)
+void reset_turn_stats(std::string save_id)
 {
-	stats &s = get_stats(side);
+	stats &s = get_stats(save_id);
 	s.turn_damage_inflicted = 0;
 	s.turn_damage_taken = 0;
 	s.turn_expected_damage_inflicted = 0;
 	s.turn_expected_damage_taken = 0;
+	s.save_id = save_id;
 }
 
-stats calculate_stats(int category, int side)
+stats calculate_stats(int category, std::string save_id)
 {
+	DBG_NG << "calculate_stats, category: " << category << " side: " << save_id << " master_stats.size: " << master_stats.size() << "\n";
 	if(category == 0) {
 		stats res;
-		for(int i = 1; i <= int(master_stats.size()); ++i) {
-			merge_stats(res,calculate_stats(i,side));
+		// We are going from last to first to include correct turn stats in result
+		for(int i = int(master_stats.size()); i > 0 ; --i) {
+			merge_stats(res,calculate_stats(i,save_id));
 		}
 
 		return res;
 	} else {
 		const size_t index = master_stats.size() - size_t(category);
-		const size_t side_index = size_t(side) - 1;
-		if(index < master_stats.size() && side_index < master_stats[index].team_stats.size()) {
-			return master_stats[index].team_stats[side_index];
+		if(index < master_stats.size() && master_stats[index].team_stats.find(save_id) != master_stats[index].team_stats.end()) {
+			return master_stats[index].team_stats[save_id];
 		} else {
 			return stats();
 		}
@@ -530,7 +615,7 @@ stats calculate_stats(int category, int side)
 config write_stats()
 {
 	config res;
-	res["mid_scenario"] = (mid_scenario ? "true" : "false");
+	res["mid_scenario"] = (mid_scenario ? "yes" : "no");
 
 	for(std::vector<scenario_stats>::const_iterator i = master_stats.begin(); i != master_stats.end(); ++i) {
 		res.add_child("scenario",i->write());
@@ -541,7 +626,7 @@ config write_stats()
 
 void write_stats(config_writer &out)
 {
-	out.write_key_val("mid_scenario", mid_scenario ? "true" : "false");
+	out.write_key_val("mid_scenario", mid_scenario ? "yes" : "no");
 
 	for(std::vector<scenario_stats>::const_iterator i = master_stats.begin(); i != master_stats.end(); ++i) {
 		out.open_child("scenario");
@@ -553,7 +638,7 @@ void write_stats(config_writer &out)
 void read_stats(const config& cfg)
 {
 	fresh_stats();
-	mid_scenario = (cfg["mid_scenario"] == "true");
+	mid_scenario = (utils::string_bool(cfg["mid_scenario"]));
 
 	const config::child_list& scenarios = cfg.get_children("scenario");
 	for(config::child_list::const_iterator i = scenarios.begin(); i != scenarios.end(); ++i) {
