@@ -1,4 +1,4 @@
-/* $Id: terrain_filter.cpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: terrain_filter.cpp 34087 2009-03-24 13:30:54Z suokko $ */
 /*
    Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -223,15 +223,15 @@ bool terrain_filter::match_internal(const map_location& loc, const bool ignore_x
 	if(!tod_type.empty()) {
 		const std::vector<std::string>& vals = utils::split(tod_type);
 		if(tod.lawful_bonus<0) {
-			if(std::find(vals.begin(),vals.end(),"chaotic") == vals.end()) {
+			if(std::find(vals.begin(),vals.end(),std::string("chaotic")) == vals.end()) {
 				return false;
 			}
 		} else if(tod.lawful_bonus>0) {
-			if(std::find(vals.begin(),vals.end(),"lawful") == vals.end()) {
+			if(std::find(vals.begin(),vals.end(),std::string("lawful")) == vals.end()) {
 				return false;
 			}
 		} else {
-			if(std::find(vals.begin(),vals.end(),"neutral") == vals.end()) {
+			if(std::find(vals.begin(),vals.end(),std::string("neutral")) == vals.end()) {
 				return false;
 			}
 		}
