@@ -1,4 +1,4 @@
-/* $Id: terrain_translation.cpp 33719 2009-03-15 22:57:51Z ilor $ */
+/* $Id: terrain_translation.cpp 34467 2009-04-04 16:04:47Z ilor $ */
 /*
    Copyright (C) 2006 - 2009 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -314,7 +314,7 @@ t_map read_game_map(const std::string& str,	std::map<int, coordinate>& starting_
 		result[x][y] = tile;
 
 		// Evaluate the separator
-		if(utils::isnewline(str[pos_separator]) || pos_separator == std::string::npos) {
+		if(pos_separator == std::string::npos || utils::isnewline(str[pos_separator])) {
 			// the first line we set the with the other lines we check the width
 			if(y == 0) {
 				// x contains the offset in the map
