@@ -1,6 +1,6 @@
-/* $Id: theme.cpp 24165 2008-02-29 23:36:38Z jhinrichs $ */
+/* $Id: theme.cpp 33185 2009-02-28 22:09:57Z ilor $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,25 +12,17 @@
    See the COPYING file for more details.
 */
 
-//! @file theme.cpp
-//!
+/** @file theme.cpp */
 
 #include "global.hpp"
 
-#include "config.hpp"
 #include "font.hpp"
 #include "gettext.hpp"
-#include "language.hpp"
+#include "hotkeys.hpp"
 #include "log.hpp"
-#include "sdl_utils.hpp"
 #include "theme.hpp"
-#include "util.hpp"
-#include "serialization/string_utils.hpp"
 #include "wml_exception.hpp"
 
-#include <cassert>
-#include <cstdlib>
-#include <sstream>
 
 #define DBG_DP LOG_STREAM(debug, display)
 #define LOG_DP LOG_STREAM(info, display)
@@ -153,10 +145,18 @@ static config& find_ref(const std::string& id, config& cfg, bool remove = false)
 	}
 
 #ifdef DEBUG
-	// to be called from gdb
-	config& find_ref(const char* id, config& cfg) {
-		return find_ref(std::string(id),cfg);
-	}
+
+// to be called from gdb
+static config& find_ref(const char* id, config& cfg) {
+	return find_ref(std::string(id),cfg);
+}
+
+namespace {
+	// avoid some compiler warnings in stricter mode.
+	static config cfg;
+	static config& result = find_ref("", cfg);
+} // namespace
+
 #endif
 
 static void expand_partialresolution(config& dst_cfg, const config& top_cfg)
@@ -267,8 +267,14 @@ static void do_resolve_rects(const config& cfg, config& resolved_config, config*
 		}
 	}
 
-theme::object::object() : location_modified_(false), loc_(empty_rect), relative_loc_(empty_rect),
-                          last_screen_(empty_rect), xanchor_(object::FIXED), yanchor_(object::FIXED)
+theme::object::object() :
+	location_modified_(false),
+	id_(),
+	loc_(empty_rect),
+	relative_loc_(empty_rect),
+	last_screen_(empty_rect),
+	xanchor_(object::FIXED),
+	yanchor_(object::FIXED)
 {
 }
 
@@ -280,7 +286,21 @@ theme::object::object(const config& cfg) :
 }
 
 theme::tborder::tborder() :
-		size(0.0)
+	size(0.0),
+	background_image(),
+	tile_image(),
+	corner_image_top_left(),
+	corner_image_bottom_left(),
+	corner_image_top_right_odd(),
+	corner_image_top_right_even(),
+	corner_image_bottom_right_odd(),
+	corner_image_bottom_right_even(),
+	border_image_left(),
+	border_image_right(),
+	border_image_top_odd(),
+	border_image_top_even(),
+	border_image_bottom_odd(),
+	border_image_bottom_even()
 {
 }
 
@@ -325,10 +345,10 @@ SDL_Rect& theme::object::location(const SDL_Rect& screen) const
 		break;
 	case TOP_ANCHORED:
 		relative_loc_.x = loc_.x;
-		relative_loc_.w = screen.w - minimum<size_t>(XDim - loc_.w,screen.w);
+		relative_loc_.w = screen.w - std::min<size_t>(XDim - loc_.w,screen.w);
 		break;
 	case BOTTOM_ANCHORED:
-		relative_loc_.x = screen.w - minimum<size_t>(XDim - loc_.x,screen.w);
+		relative_loc_.x = screen.w - std::min<size_t>(XDim - loc_.x,screen.w);
 		relative_loc_.w = loc_.w;
 		break;
 	case PROPORTIONAL:
@@ -346,10 +366,10 @@ SDL_Rect& theme::object::location(const SDL_Rect& screen) const
 		break;
 	case TOP_ANCHORED:
 		relative_loc_.y = loc_.y;
-		relative_loc_.h = screen.h - minimum<size_t>(YDim - loc_.h,screen.h);
+		relative_loc_.h = screen.h - std::min<size_t>(YDim - loc_.h,screen.h);
 		break;
 	case BOTTOM_ANCHORED:
-		relative_loc_.y = screen.h - minimum<size_t>(YDim - loc_.y,screen.h);
+		relative_loc_.y = screen.h - std::min<size_t>(YDim - loc_.y,screen.h);
 		relative_loc_.h = loc_.h;
 		break;
 	case PROPORTIONAL:
@@ -360,10 +380,10 @@ SDL_Rect& theme::object::location(const SDL_Rect& screen) const
 		assert(false);
 	}
 
-	relative_loc_.x = minimum<int>(relative_loc_.x,screen.w);
-	relative_loc_.w = minimum<int>(relative_loc_.w,screen.w - relative_loc_.x);
-	relative_loc_.y = minimum<int>(relative_loc_.y,screen.h);
-	relative_loc_.h = minimum<int>(relative_loc_.h,screen.h - relative_loc_.y);
+	relative_loc_.x = std::min<int>(relative_loc_.x,screen.w);
+	relative_loc_.w = std::min<int>(relative_loc_.w,screen.w - relative_loc_.x);
+	relative_loc_.y = std::min<int>(relative_loc_.y,screen.h);
+	relative_loc_.h = std::min<int>(relative_loc_.h,screen.h - relative_loc_.y);
 
 	return relative_loc_;
 }
@@ -409,18 +429,25 @@ void theme::object::modify_location(std::string rect_str, SDL_Rect ref_rect){
 	modify_location(rect);
 }
 
-theme::label::label()
+theme::label::label() :
+	text_(),
+	icon_(),
+	font_(),
+	font_rgb_set_(false),
+	font_rgb_(DefaultFontRGB)
 {}
 
-theme::label::label(const config& cfg)
-      : object(cfg), text_(cfg["prefix"].str() + cfg["text"].str() + cfg["postfix"].str()),
-	icon_(cfg["icon"]), font_(atoi(cfg["font_size"].c_str()))
+theme::label::label(const config& cfg) :
+	object(cfg),
+	text_(cfg["prefix"].str() + cfg["text"].str() + cfg["postfix"].str()),
+	icon_(cfg["icon"]),
+	font_(atoi(cfg["font_size"].c_str())),
+	font_rgb_set_(false),
+	font_rgb_(DefaultFontRGB)
 {
 	if(font_ == 0)
 		font_ = DefaultFontSize;
 
-	font_rgb_ = DefaultFontRGB;
-	font_rgb_set_ = false;
 	if(cfg["font_rgb"].size()){
 	std::vector<std::string> rgb_vec = utils::split(cfg["font_rgb"]);
 	  if(3 <= rgb_vec.size()){
@@ -445,11 +472,14 @@ theme::label::label(const config& cfg)
 	}
 }
 
-theme::status_item::status_item(const config& cfg)
-        : object(cfg),
-		  prefix_(cfg["prefix"].str() + cfg["prefix_literal"].str()),
-		  postfix_(cfg["postfix_literal"].str() + cfg["postfix"].str()),
-          font_(atoi(cfg["font_size"].c_str()))
+theme::status_item::status_item(const config& cfg) :
+	object(cfg),
+	prefix_(cfg["prefix"].str() + cfg["prefix_literal"].str()),
+	postfix_(cfg["postfix_literal"].str() + cfg["postfix"].str()),
+	label_(),
+	font_(atoi(cfg["font_size"].c_str())),
+	font_rgb_set_(false),
+	font_rgb_(DefaultFontRGB)
 {
 	if(font_ == 0)
 		font_ = DefaultFontSize;
@@ -459,8 +489,6 @@ theme::status_item::status_item(const config& cfg)
 		label_ = label(*label_child);
 	}
 
-	font_rgb_ = DefaultFontRGB;
-	font_rgb_set_ = false;
 	if(cfg["font_rgb"].size()){
 	  std::vector<std::string> rgb_vec = utils::split(cfg["font_rgb"]);
 	  if(3 <= rgb_vec.size()){
@@ -488,18 +516,43 @@ theme::status_item::status_item(const config& cfg)
 theme::panel::panel(const config& cfg) : object(cfg), image_(cfg["image"])
 {}
 
-theme::menu::menu() : context_(false)
+theme::menu::menu() :
+	object(),
+	context_(false),
+	title_(),
+	tooltip_(),
+	image_(),
+	type_(),
+	items_()
 {}
 
-theme::menu::menu(const config& cfg) : object(cfg), context_(cfg["is_context_menu"] == "true"),
+theme::menu::menu(const config& cfg) : object(cfg),
+                                       context_(utils::string_bool(cfg["is_context_menu"])),
                                        title_(cfg["title"].str() + cfg["title_literal"].str()),
 									   tooltip_(cfg["tooltip"]),
-						image_(cfg["image"]), type_(cfg["type"]),
-						items_(utils::split(cfg["items"]))
-{}
+									   image_(cfg["image"]), type_(cfg["type"]),
+									   items_(utils::split(cfg["items"]))
+{
+	if (utils::string_bool(cfg["auto_tooltip"]) && tooltip_.empty() && items_.size() == 1) {
+		tooltip_ = hotkey::get_hotkey(items_[0]).get_description();
+	} else if (utils::string_bool(cfg["tooltip_name_prepend"]) && items_.size() == 1) {
+		tooltip_ = hotkey::get_hotkey(items_[0]).get_description() + "\n" + tooltip_;
+	}
+}
 
 theme::theme(const config& cfg, const SDL_Rect& screen) :
-	theme_reset_("theme_reset")
+	theme_reset_("theme_reset"),
+	cur_theme(),
+	cfg_(),
+	panels_(),
+	labels_(),
+	menus_(),
+	context_(),
+	status_(),
+	main_map_(),
+	mini_map_(),
+	unit_image_(),
+	border_()
 {
 	config tmp;
 	expand_partialresolution(tmp, cfg);
@@ -534,12 +587,13 @@ bool theme::set_resolution(const SDL_Rect& screen)
 	}
 
 	if(current == resolutions.end()) {
-		if(!resolutions.empty())
+		if(!resolutions.empty()) {
 			LOG_STREAM(err, display) << "No valid resolution found\n";
+		}
 		return false;
 	}
 
-	std::map<std::string,std::string> title_stash;	
+	std::map<std::string,std::string> title_stash;
 	std::vector<theme::menu>::iterator m;
 	for (m = menus_.begin(); m != menus_.end(); ++m) {
 		if (!m->title().empty() && !m->get_id().empty())
@@ -556,11 +610,11 @@ bool theme::set_resolution(const SDL_Rect& screen)
 
 	for (m = menus_.begin(); m != menus_.end(); ++m) {
 		if (title_stash.find(m->get_id()) != title_stash.end())
-			m->set_title(title_stash[m->get_id()]); 
+			m->set_title(title_stash[m->get_id()]);
 	}
-	
+
 	theme_reset_.notify_observers();
-	
+
 	return result;
 }
 
@@ -663,7 +717,7 @@ void theme::set_object_location(theme::object& element, std::string rect_str, st
 }
 
 void theme::modify(const config* cfg){
-	std::map<std::string,std::string> title_stash;	
+	std::map<std::string,std::string> title_stash;
 	std::vector<theme::menu>::iterator m;
 	for (m = menus_.begin(); m != menus_.end(); ++m) {
 		if (!m->title().empty() && !m->get_id().empty())
@@ -698,7 +752,7 @@ void theme::modify(const config* cfg){
 	}
 	for (m = menus_.begin(); m != menus_.end(); ++m) {
 		if (title_stash.find(m->get_id()) != title_stash.end())
-			m->set_title(title_stash[m->get_id()]); 
+			m->set_title(title_stash[m->get_id()]);
 	}
 }
 
@@ -759,8 +813,8 @@ theme::menu* theme::refresh_title(const std::string& id, const std::string& new_
 	theme::menu* res = NULL;
 
 	for (std::vector<theme::menu>::iterator m = menus_.begin(); m != menus_.end(); ++m){
-		if (m->get_id() == id) { 
-			res = &(*m); 
+		if (m->get_id() == id) {
+			res = &(*m);
 			res->set_title(new_title);
 		}
 	}
