@@ -1,6 +1,6 @@
-/* $Id: thread.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: thread.hpp 31859 2009-01-01 10:28:26Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -18,6 +18,11 @@
 #include "SDL.h"
 #include "SDL_thread.h"
 
+#include <list>
+
+#include <boost/scoped_ptr.hpp>
+#include <boost/smart_ptr.hpp>
+
 // Threading primitives wrapper for SDL_Thread.
 //
 // This module defines primitives for wrapping C++ around SDL's threading
@@ -190,6 +195,12 @@ public:
 	virtual ACTION process() = 0;
 };
 
+class async_operation;
+
+typedef boost::shared_ptr<async_operation> async_operation_ptr;
+
+typedef std::list<async_operation_ptr> active_operation_list;
+
 //class which defines an asynchronous operation. Objects of this class are accessed from
 //both the worker thread and the calling thread, and so it has 'strange' allocation semantics.
 //It is allocated by the caller, and generally deleted by the caller. However, in some cases
@@ -205,11 +216,15 @@ public:
 
 	enum RESULT { COMPLETED, ABORTED };
 
-	async_operation() : 
-		aborted_(false), finished_(), finishedVar_(false), mutex_() {}
+	async_operation() :
+		thread_(), aborted_(false), finished_(), finishedVar_(false), mutex_()
+	{
+		while (!active_.empty() && active_.front().unique())
+			active_.pop_front();
+	}
 	virtual ~async_operation() {}
 
-	RESULT execute(waiter& wait);
+	RESULT execute(async_operation_ptr this_ptr, waiter& wait);
 
 	mutex& get_mutex() { return mutex_; }
 
@@ -225,32 +240,13 @@ public:
 	bool is_aborted() const { return aborted_; }
 
 private:
+	boost::scoped_ptr<thread> thread_;
 	bool aborted_;
 	condition finished_;
 	bool finishedVar_;
 	mutex mutex_;
-};
 
-//T should be a type derived from async_operation
-template<typename T>
-class async_operation_holder
-{
-public:
-	explicit async_operation_holder(T* op) : op_(op)
-	{}
-
-	~async_operation_holder() {
-		//it's okay to call is_aborted() without the mutex here,
-		//because we are in the calling thread, not the worker thread
-		if(op_->is_aborted() == false) {
-			delete op_;
-		}
-	}
-
-	T& operation() const { return *op_; }
-
-private:
-	T* const op_;
+	static active_operation_list active_;
 };
 
 }
