@@ -1,6 +1,6 @@
-/* $Id: cutter.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: cutter.cpp 31858 2009-01-01 10:27:41Z mordante $ */
 /*
-   Copyright (C) 2004 - 2008 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2004 - 2009 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -12,12 +12,12 @@
    See the COPYING file for more details.
 */
 
-//! @file tools/cutter.cpp
-//! Standalone-Utility for images / tiles
+/**
+ * @file tools/cutter.cpp
+ * Standalone-Utility for images / tiles
+ */
 
-#include "../sdl_utils.hpp"
 #include "../game_config.hpp"
-#include "exploder_utils.hpp"
 #include "exploder_composer.hpp"
 
 #include "SDL_image.h"
