@@ -1,6 +1,6 @@
-/* $Id: exploder_cutter.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: exploder_cutter.cpp 31859 2009-01-01 10:28:26Z mordante $ */
 /*
-   Copyright (C) 2004 - 2008 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2004 - 2009 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -14,15 +14,14 @@
 
 #include "exploder_cutter.hpp"
 #include "filesystem.hpp"
-#include "sdl_utils.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/preprocessor.hpp"
-#include "serialization/string_utils.hpp"
 #include "SDL_image.h"
 
-cutter::cutter() : verbose_(false)
+cutter::cutter()
+	: masks_()
+	, verbose_(false)
 {
-
 }
 
 const config cutter::load_config(const std::string &filename)
