@@ -1,4 +1,4 @@
-/* $Id: unit_frame.hpp 32322 2009-01-26 19:55:53Z boucman $ */
+/* $Id: unit_frame.hpp 35578 2009-05-11 21:45:03Z boucman $ */
 /*
    Copyright (C) 2006 - 2009 by Jeremy Rosen <jeremy.rosen@enst-bretagne.fr>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -67,8 +67,8 @@ typedef progressive_<double> progressive_double;
 class frame_parameters{
 	public:
 	frame_parameters():
-	image(""),
-	image_diagonal(""),
+	image(),
+	image_diagonal(),
 	image_mod(""),
 	halo(""),
 	halo_x(0),
@@ -107,6 +107,8 @@ class frame_parameters{
 	int x;
 	int y;
 	int drawing_layer;
+	bool in_hex;
+	bool diagonal_in_hex;
 } ;
 /**
  * keep most parameters in a separate class to simplify handling of large
@@ -178,7 +180,6 @@ class frame_builder {
 		progressive_int x_;
 		progressive_int y_;
 		progressive_int drawing_layer_;
-
 };
 
 /** Describe a unit's animation sequence. */
