@@ -1,4 +1,4 @@
-/* $Id: unit_types.cpp 33687 2009-03-15 16:50:42Z mordante $ */
+/* $Id: unit_types.cpp 35616 2009-05-13 03:46:28Z alink $ */
 /*
    Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -1278,26 +1278,29 @@ unit_type& unit_type_data::unit_type_map_wrapper::build_unit_type(const std::str
 
     DBG_UT << "Building unit type " << ut->first << ", level " << status << "\n";
 
-    const config& unit_cfg = find_config(key);
-
     switch (status){
-        case unit_type::CREATED:
+        case unit_type::CREATED: {
+			const config& unit_cfg = find_config(key);
             ut->second.set_config(unit_cfg);
             ut->second.build_created(unit_cfg, movement_types_, races_, unit_traits_);
             break;
-        case unit_type::HELP_INDEX:
+        }
+        case unit_type::HELP_INDEX: {
             //build the stuff that is needed to feed the help index
-            if ( (ut->second.build_status() == unit_type::NOT_BUILT) || (ut->second.build_status() == unit_type::CREATED) )
-                ut->second.build_help_index(unit_cfg, movement_types_, races_, unit_traits_);
-
-            add_advancefrom(unit_cfg);
+            const config& unit_cfg = find_config(key);
+            if ( (ut->second.build_status() == unit_type::NOT_BUILT) || (ut->second.build_status() == unit_type::CREATED) ) {
+				ut->second.build_help_index(unit_cfg, movement_types_, races_, unit_traits_);
+            }
+			add_advancefrom(unit_cfg);
             break;
+        }
         case unit_type::WITHOUT_ANIMATIONS:
-        case unit_type::FULL:
+        case unit_type::FULL: {
             if ( (ut->second.build_status() == unit_type::NOT_BUILT) ||
                 (ut->second.build_status() == unit_type::CREATED) ||
                 (ut->second.build_status() == unit_type::HELP_INDEX) )
             {
+				const config& unit_cfg = find_config(key);
                 ut->second.build_full(unit_cfg, movement_types_, races_, unit_traits_);
 
                 if ( (ut->second.build_status() == unit_type::NOT_BUILT) ||
@@ -1305,6 +1308,7 @@ unit_type& unit_type_data::unit_type_map_wrapper::build_unit_type(const std::str
                     add_advancefrom(unit_cfg);
             }
             break;
+        }
         default:
             break;
     }
