@@ -1,6 +1,6 @@
-/* $Id: combo.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: combo.cpp 31858 2009-01-01 10:27:41Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,28 +12,27 @@
    See the COPYING file for more details.
 */
 
+#define GETTEXT_DOMAIN "wesnoth-lib"
+
 #include "global.hpp"
 
 #include "widgets/combo.hpp"
 #include "display.hpp"
 #include "show_dialog.hpp"
-#include "video.hpp"
 
-namespace {
-
-#ifdef USE_TINY_GUI
-	const std::string empty_combo_label = "           ";
-#else
-	const std::string empty_combo_label = "";
-#endif
-
-}
 
 namespace gui {
 
-const int font_size = font::SIZE_SMALL;
-const int horizontal_padding = 10;
-const int vertical_padding = 10;
+#ifdef USE_TINY_GUI
+	const std::string combo::empty_combo_label = "           ";
+#else
+	const std::string combo::empty_combo_label = "";
+#endif
+
+
+const int combo::font_size = font::SIZE_SMALL;
+const int combo::horizontal_padding = 10;
+const int combo::vertical_padding = 10;
 
 combo::combo(display& disp, const std::vector<std::string>& items)
 	: button(disp.video(), items.empty() ? empty_combo_label : items[0]),
@@ -82,13 +81,18 @@ void combo::set_selected(int val)
 	oldSelected_ = selected_;
 }
 
-void combo::process_event()
+void combo::make_drop_down_menu()
 {
-	if (!pressed())
-		return;
 	SDL_Rect const &loc = location();
 	set_selected_internal(gui::show_dialog(*disp_, NULL, "", "", gui::MESSAGE, &items_,
 	                                       NULL, "", NULL, -1, NULL, loc.x, loc.y + loc.h));
 }
 
+void combo::process_event()
+{
+	if (!pressed())
+		return;
+	make_drop_down_menu();
+}
+
 }
