@@ -1,6 +1,6 @@
-/* $Id: combo.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: combo.hpp 31859 2009-01-01 10:28:26Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -36,11 +36,16 @@ public:
 protected:
 	virtual void process_event();
 
+	void make_drop_down_menu();
 private:
 	void set_selected_internal(int val);
 	std::vector<std::string> items_;
 	int selected_, oldSelected_;
 	display* disp_;
+	static const std::string empty_combo_label;
+	static const int font_size;
+	static const int horizontal_padding;
+	static const int vertical_padding;
 }; //end class combo
 
 }
