@@ -1,6 +1,6 @@
-/* $Id: label.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: label.cpp 31858 2009-01-01 10:27:41Z mordante $ */
 /*
-   Copyright (C) 2004 - 2008 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2004 - 2009 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -12,11 +12,12 @@
    See the COPYING file for more details.
 */
 
+#define GETTEXT_DOMAIN "wesnoth-lib"
+
 #include "global.hpp"
 
 #include "widgets/label.hpp"
 #include "marked-up_text.hpp"
-#include "video.hpp"
 
 namespace gui {
 
@@ -68,7 +69,9 @@ const SDL_Color& label::get_colour() const
 
 void label::draw_contents()
 {
-	font::draw_text(&video(), location(), size_, get_colour(), text_, location().x, location().y);
+	const SDL_Rect& loc = location();
+	if (!text_.empty() && loc.w > 0 && loc.h > 0)
+		font::draw_text(&video(), loc, size_, get_colour(), text_, loc.x, loc.y);
 }
 
 void label::update_label_size()
