@@ -1,6 +1,6 @@
-/* $Id: widget.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: widget.hpp 31858 2009-01-01 10:27:41Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -95,6 +95,10 @@ protected:
 	virtual void handle_event(SDL_Event const &/*event*/) {}
 	bool focus_;		// Should user input be ignored?
 
+	bool mouse_locked() const;
+
+	void aquire_mouse_lock();
+	void free_mouse_lock();
 private:
 	void volatile_draw();
 	void volatile_undraw();
@@ -118,6 +122,9 @@ private:
 	int help_string_;
 	std::string id_;
 
+	bool mouse_lock_local_;
+	static bool mouse_lock_;
+
 	friend class scrollpane;
 	friend class dialog;
 };
